@@ -68,6 +68,7 @@ gw = {
   autocompletion: [],
   focus: false,
   lock: 0,
+  needKeyPress: {},
   
   log: function(msg)
   {
@@ -79,6 +80,13 @@ gw = {
     }
   },
   
+  getFormId: function(id)
+  {
+    var pos = id.indexOf(".");
+    if (pos > 0)
+      return id.substr(0, pos);
+  },
+  
   load: function(lib)
   {
     var elt, src;
@@ -115,12 +123,16 @@ gw = {
     var newDiv = oldDiv.cloneNode(false);
     newDiv.innerHTML = html;
     oldDiv.parentNode.replaceChild(newDiv, oldDiv);
+    gw.onFocus();
   },
   
   setOuterHtml: function(id, html)
   {
     if ($(id))
+    {
       $(id).outerHTML = html;
+      gw.onFocus();
+    }
     else
       gw.log('setOuterHtml: ' + id + '? ' + html);
   },
@@ -165,17 +177,19 @@ gw = {
     }
   },
   
-  saveFocus: function() {
+  saveFocus: function()
+  {
     var active = document.activeElement.id;
     var selection;
     
     if (active)
       selection = gw.getSelection($(active));
-      
+    
     return [active, selection];
   },
   
-  restoreFocus: function(save) {
+  restoreFocus: function(save)
+  {
     var elt;
     
     if (save[0])
@@ -185,6 +199,7 @@ gw = {
       {
         elt.focus();
         gw.setSelection(elt, save[1]);
+        gw.onFocus();
       }
     }
     //else
@@ -263,7 +278,8 @@ gw = {
         
         if (!gw.focus)
           gw.restoreFocus(save);
-        
+          
+        gw.onFocus();
       }
       
       if (after)
@@ -387,7 +403,7 @@ gw = {
   onFocus: function()
   {
     var elt = document.activeElement;
-    var id;
+    var id, pos, win;
     
     if (gw._focusTimeout)
     {
@@ -395,6 +411,9 @@ gw = {
       gw._focusTimeout = undefined;
     }
     
+    while (elt && (!elt.id || elt.id.indexOf(':') >= 0))
+      elt = elt.parentNode;
+    
     if (elt && elt.id)
     {
       id = elt.id;
@@ -405,19 +424,37 @@ gw = {
         return;
       }
       
-      gw.active = id;
-      gw.update('', '#focus', id);
-      //console.log('onFocus: ' + id);
+      if (gw.active != id)
+      {
+        gw.active = id;
+        gw.update('', '#focus', id);
+      }
     }
-    else
+    else if (gw.active)
     {
+      gw.active = undefined;
       gw._focusTimeout = setTimeout(function() 
         {
           gw._focusTimeout = undefined;
-          //console.log('onFocus: -');
           gw.update('', '#focus', '');
         }, 50);
     }
+    
+    if (elt && elt != document.body)
+    {
+      pos = gw.getPos(elt);
+      $('gw-focus').style.top = pos.top + 'px';
+      $('gw-focus').style.left = pos.left + 'px';
+      $('gw-focus').style.width = pos.width + 'px';
+      $('gw-focus').style.height = pos.height + 'px';
+      win = gw.getWindow(elt);
+      $('gw-focus').style.zIndex = win ? (parseInt(win.style.zIndex) + 2) : 2;
+      $('gw-focus').style.display = 'block';
+    }
+    else
+    {
+      $('gw-focus').style.display = '';
+    }
   },
   
   setFocus: function(id)
@@ -429,6 +466,7 @@ gw = {
       elt.focus();
       gw.selection = undefined;
       gw.focus = true;
+      gw.onFocus();
     }
   },
   
@@ -521,6 +559,18 @@ gw = {
     
     return { found: found, left: left, top: top, width: width, height: height, right: left + width, bottom: top + height };
   },
+  
+  getWindow: function(elt)
+  {
+    while (!elt.hasClass('gw-window'))
+    {
+      elt = elt.parentNode;
+      if (elt == document.body || !elt)
+        return null;
+    }
+      
+    return elt;
+  },
 
   copy: function(elt)
   {
@@ -682,6 +732,12 @@ gw = {
       gw.window.refresh();
     },
     
+    translate: function(id, x, y)
+    {
+      $(id).style.transform = 'translate(' + (x|0) + 'px,' + (y|0) + 'px)';
+      gw.onFocus();
+    },
+    
     popup: function(id, resizable, control, transient, minw, minh)
     {
       var pos;
@@ -705,9 +761,9 @@ gw = {
         /*$(id).style.left = pos.left + 'px';
         $(id).style.top = pos.bottom + 'px';*/
         if (transient)
-          $(id).style.transform = 'translate(' + (pos.left|0) + 'px,' + (pos.top|0) + 'px)';
+          gw.window.translate(id, pos.left, pos.top);
         else
-          $(id).style.transform = 'translate(' + (pos.left|0) + 'px,' + (pos.bottom|0) + 'px)';
+          gw.window.translate(id, pos.left, pos.bottom);
       }
       
       $(id).gw_resizable = resizable;
@@ -754,6 +810,7 @@ gw = {
       {
         gw.restoreFocus($(id).gw_focus);
         $(id).gw_focus = undefined;
+        gw.onFocus();
       }
       
       $(id).gw_minw = $(id).gw_minh = undefined;
@@ -768,7 +825,7 @@ gw = {
       {
         if ($(gw.windows[i]))
         {
-          zi = 11 + i * 2;
+          zi = 11 + i * 4;
           if ($(gw.windows[i]).style.zIndex != zi)
             $(gw.windows[i]).style.zIndex = zi;
           i++;
@@ -787,6 +844,8 @@ gw = {
       }
       else
         gw.window.updateTitleBars();
+        
+      gw.onFocus();
     },
     
     updateTitleBars: function()
@@ -820,7 +879,7 @@ gw = {
       gw.windows.push(id);
       
       for (i = 0; i < gw.windows.length; i++)
-        $(gw.windows[i]).style.zIndex = 11 + i * 2;
+        $(gw.windows[i]).style.zIndex = 11 + i * 4;
         
       gw.window.updateTitleBars();
       $(id).focus();
@@ -837,24 +896,26 @@ gw = {
       {
         if ($(gw.windows[i]).gw_modal)
         {
-          gw.window.zIndex = 10 + i * 2;
-          elt.style.zIndex = 10 + i * 2;
+          gw.window.zIndex = 10 + i * 4;
+          elt.style.zIndex = 10 + i * 4;
           elt.style.display = 'block';
           /*if ($(gw.windows[i]).gw_popup)
             elt.style.opacity = '0';
           else
             elt.style.opacity = '';*/
+          gw.onFocus();
           return;
         }
       }
       
       gw.window.zIndex = 0;
       elt.style.display = 'none';
+      gw.onFocus();
     },
     
     center: function(id)
     {
-      $(id).style.transform = 'translate(' + ((window.innerWidth - $(id).offsetWidth) / 2 | 0) + 'px,' + ((window.innerHeight - $(id).offsetHeight) / 2 | 0) + 'px)';
+      gw.window.translate(id, (window.innerWidth - $(id).offsetWidth) / 2, (window.innerHeight - $(id).offsetHeight) / 2);
       gw.window.updateGeometry(id);
     },
     
@@ -1009,6 +1070,7 @@ gw = {
     {
       var b = $(id).getBoundingClientRect();
       gw.update(id, '#geometry', [ b.left + 'px', b.top + 'px', b.width + 'px', b.height + 'px', gw.window.isMaximized(id)]);
+      gw.onFocus();
     },
     
     onUp: function(e)
@@ -1061,7 +1123,7 @@ gw = {
           {
             elt.style.width = w + 'px';
             //elt.style.left = x + 'px'; 
-            elt.style.transform = 'translate(' + x + 'px,' + y + 'px)';
+            gw.window.translate(c.id, x, y);
             //c.x = x;
           }
         }
@@ -1074,7 +1136,7 @@ gw = {
           {
             elt.style.height = h + 'px';
             //elt.style.top = y + 'px'; 
-            elt.style.transform = 'translate(' + x + 'px,' + y + 'px)';
+            gw.window.translate(c.id, x, y);
           }
         }
     
@@ -1085,7 +1147,7 @@ gw = {
       {
         /*elt.style.left = (Math.max(0, c.x + c.cx - e.clientX)) + 'px';
         elt.style.top = (Math.max(0, c.y + c.cy - e.clientY)) + 'px';*/
-        elt.style.transform = 'translate(' + (Math.max(0, c.x + c.cx - e.clientX)) + 'px,' + (Math.min(window.innerHeight - elt.firstElementChild.offsetHeight, Math.max(0, c.y + c.cy - e.clientY))) + 'px)';
+        gw.window.translate(c.id, Math.max(0, c.x + c.cx - e.clientX), Math.min(window.innerHeight - elt.firstElementChild.offsetHeight, Math.max(0, c.y + c.cy - e.clientY)));
         return;
       }
     
@@ -1127,23 +1189,36 @@ gw = {
       var i;
       var tr;
       
-      if (end < start)
+      if ($(id).hasClass('gw-table'))
       {
-        i = start;
-        start = end;
-        end = i;
+        if (end < start)
+        {
+          i = start;
+          start = end;
+          end = i;
+        }
+        
+        for (i = start; i <= end; i++)
+        {
+          tr = $(id + ':' + i);
+          if (checked)
+            tr.addClass('gw-selected');
+          else
+            tr.removeClass('gw-selected');
+        }
+          
+        gw.update(id, '!' + start + ':' + end, checked);
       }
-      
-      for (i = start; i <= end; i++)
+      else
       {
-        tr = $(id + ':' + i);
+        tr = $(id + ':' + start);
         if (checked)
           tr.addClass('gw-selected');
         else
           tr.removeClass('gw-selected');
-      }
         
-      gw.update(id, '!' + start + ':' + (end - start + 1), checked);
+        gw.update(id, '!' + start, checked);
+      }
     },
   
     select: function(id, row, event)
@@ -1185,17 +1260,20 @@ gw = {
         }
         
         for (i = start; i <= end; i++)
-          $(id + '!' + i).checked = checked;
+          $(id + ':c:' + i).checked = checked;
+      
+        gw.update(id, '!' + start + ':' + end, checked);
       }
       else
-        $(id + '!' + start).checked = checked;
-        
-      gw.update(id, '!' + start + ':' + end, checked);
+      {
+        $(id + ':c:' + start).checked = checked;
+        gw.update(id, '!' + start, checked);
+      }
     },
   
     check: function(id, row, event)
     {
-      var elt = $(id + '!' + row);
+      var elt = $(id + ':c:' + row);
       var checked = !elt.checked;
       var last = $(id).gw_current;
       var len;
@@ -1743,41 +1821,51 @@ gw = {
       null);
   },
   
-  onKeyDown: function(id, event)
+  onKeyDown: function(event)
   {
+    var elt;
+    var id;
+    
+    if (event.bubbles && gw.shortcuts)
+    {
+      var shortcut = gw.makeShortcut(event);
+      if (gw.shortcuts[shortcut])
+      {
+        //gw.log('shortcut -> ' + shortcut);
+        gw.sendKeyPress(event, '');
+        event.preventDefault();
+        return;
+      }
+    }
+    
     if (!event.bubbles)
       return;
     
-    if (gw.active) id = gw.active;
-    var elt = $(id);
-    while (elt)
+    id = '';
+    elt = document.activeElement;
+    while (elt) // TODO: stop at window?
     {
       id = elt.id;
       if (id && id.indexOf(':') < 0)
         break;
       elt = elt.parentNode;
-      id = '';
     }
   
-    gw.sendKeyPress(event, id);
+    if (id && (gw.needKeyPress[id] != undefined || gw.needKeyPress[gw.getFormId(id)]))
+      gw.sendKeyPress(event, id);
   },
   
-  onShortcut: function(event)
+  onload: function()
   {
-    if (event.bubbles && gw.shortcuts)
-    {
-      gw.sendKeyPress(event, '');
-      var shortcut = gw.makeShortcut(event);
-      if (gw.shortcuts[shortcut])
-      {
-        gw.log('shortcut -> ' + shortcut);
-        event.preventDefault();
-      }
-    }
-  },
+    document.body.addEventListener('focusin', gw.onFocus);
+    document.body.addEventListener('focusout', gw.onFocus);
+    gw.raise(null, 'open');
+    document.body.style.opacity = '1';
+    document.body.style.pointerEvents = 'auto';
+  }
 }
 
-document.onkeydown = gw.onShortcut;
-document.addEventListener('focusin', gw.onFocus);
-document.addEventListener('focusout', gw.onFocus);
+document.onkeydown = gw.onKeyDown;
+
+
 
