@@ -36,20 +36,27 @@
 DECLARE_EVENT(EVENT_Change);
 DECLARE_EVENT(EVENT_Activate);
 DECLARE_EVENT(EVENT_Click);
+DECLARE_EVENT(EVENT_Cursor);
 
 
-static void cb_change(gTextBox *sender)
+void CB_textbox_change(gTextBox *sender)
 {
 	CWIDGET *_object = GetObject((gControl*)sender);
 	GB.Raise(THIS, EVENT_Change, 0);
 }
 
-static void cb_activate(gTextBox *sender)
+void CB_textbox_activate(gTextBox *sender)
 {
 	CWIDGET *_object = GetObject((gControl*)sender);
 	GB.Raise(THIS, EVENT_Activate, 0);
 }
 
+void CB_textbox_cursor(gTextBox *sender)
+{
+	CWIDGET *_object = GetObject((gControl*)sender);
+	GB.Raise(THIS, EVENT_Cursor, 0);
+}
+
 /***************************************************************************
 
 	TextBox
@@ -66,8 +73,6 @@ static void cb_activate(gTextBox *sender)
 BEGIN_METHOD(TextBox_new, GB_OBJECT parent)
 
 	InitControl(new gTextBox(CONTAINER(VARG(parent))), (CWIDGET*)THIS);
-	TEXTBOX->onChange = cb_change;
-	TEXTBOX->onActivate = cb_activate;
 	
 END_METHOD
 
@@ -257,7 +262,7 @@ END_METHOD
 #undef THIS
 #define THIS ((CCOMBOBOX *)_object)
 
-static void cb_click(gComboBox *sender)
+void CB_combobox_click(gComboBox *sender)
 {
 	CWIDGET *_object = GetObject((gControl*)sender);
 	if (THIS->click)
@@ -272,10 +277,6 @@ BEGIN_METHOD(ComboBox_new, GB_OBJECT parent)
 
 	InitControl(new gComboBox(CONTAINER(VARG(parent))), (CWIDGET*)THIS);
 	
-	COMBOBOX->onClick = cb_click;
-	COMBOBOX->onChange = cb_change;
-	COMBOBOX->onActivate = cb_activate;
-
 END_METHOD
 
 
@@ -295,6 +296,12 @@ BEGIN_METHOD_VOID(ComboBox_Popup)
 
 END_METHOD
 
+BEGIN_METHOD_VOID(ComboBox_Close)
+
+	COMBOBOX->popdown();
+
+END_METHOD
+
 
 BEGIN_METHOD(ComboBox_get, GB_INTEGER index)
 
@@ -476,6 +483,7 @@ GB_DESC CTextBoxDesc[] =
 
 	GB_EVENT("Change", 0, 0, &EVENT_Change),
 	GB_EVENT("Activate", 0, 0, &EVENT_Activate),
+	GB_EVENT("Cursor", 0, 0, &EVENT_Cursor),
 	
 	TEXTBOX_DESCRIPTION,
 
@@ -500,6 +508,7 @@ GB_DESC CComboBoxDesc[] =
 	GB_METHOD("_new", 0, ComboBox_new, "(Parent)Container;"),
 	GB_METHOD("_get", ".ComboBox.Item", ComboBox_get, "(Index)i"),
 	GB_METHOD("Popup", 0, ComboBox_Popup, 0),
+	GB_METHOD("Close", 0, ComboBox_Close, 0),
 	GB_METHOD("Clear", 0, TextBox_Clear, 0),
 	GB_METHOD("Insert", 0, TextBox_Insert, "(Text)s"),
 	GB_METHOD("Add", 0, ComboBox_Add, "(Item)s[(Index)i]"),
@@ -531,6 +540,7 @@ GB_DESC CComboBoxDesc[] =
 	
 	GB_EVENT("Change", 0, 0, &EVENT_Change),
 	GB_EVENT("Activate", 0, 0, &EVENT_Activate),
+	GB_EVENT("Cursor", 0, 0, &EVENT_Cursor),
 	GB_EVENT("Click", 0, 0, &EVENT_Click),
 
 	COMBOBOX_DESCRIPTION,
