@@ -49,6 +49,7 @@
 
 #ifdef GTK3
 static GtkApplication *_app;
+GtkStyleProvider *_tooltip_css = NULL;
 #endif
 
 static bool _debug_keypress = false;
@@ -257,6 +258,7 @@ static void gambas_handle_event(GdkEvent *event)
 		goto __HANDLE_EVENT;
 
 	widget = gtk_get_event_widget(event);
+	
 	if (!widget)
 		goto __HANDLE_EVENT;
 	
@@ -341,7 +343,7 @@ static void gambas_handle_event(GdkEvent *event)
 
 		//if (GTK_IS_WINDOW(widget))
 		{
-			control = gt_get_control(widget);
+			//control = gt_get_control(widget);
 			if (control)
 				gApplication::setActiveControl(control, event->focus_change.in);
 			else if (event->focus_change.in)
@@ -555,7 +557,11 @@ __FOUND_WIDGET:
 
 			control = save_control;
 
-			bool menu = false;
+#if GTK_CHECK_VERSION(3, 4, 0)
+			bool menu = gdk_event_triggers_context_menu(event);
+#else
+			bool menu = (event->button.button == 3) && (event->type == GDK_BUTTON_PRESS);
+#endif
 
 			if (event->type != GDK_BUTTON_RELEASE)
 			{
@@ -573,14 +579,11 @@ __FOUND_WIDGET:
 
 		__BUTTON_TRY_PROXY:
 		
-			if (!control->isDesign() && !control->isEnabled())
-				goto __HANDLE_EVENT;
-
 			cancel = false;
 
-			if (control->onMouseEvent)
+			if (control->isDesign() || control->isEnabled())
 			{
-				if (event->type == GDK_BUTTON_PRESS || control->canRaise(control, type))
+				if (event->type == GDK_BUTTON_PRESS || CB_control_can_raise(control, type))
 				{
 					control->getScreenPos(&xc, &yc);
 					xs = (int)event->button.x_root;
@@ -597,16 +600,16 @@ __FOUND_WIDGET:
 						case GDK_BUTTON_PRESS:
 							gMouse::setControl(control);
 							gMouse::setStart(x, y);
-							cancel = control->onMouseEvent(control, gEvent_MousePress);
+							cancel = CB_control_mouse(control, gEvent_MousePress);
 							break;
 
 						case GDK_2BUTTON_PRESS:
-							cancel = control->onMouseEvent(control, gEvent_MouseDblClick);
+							cancel = CB_control_mouse(control, gEvent_MouseDblClick);
 							break;
 
 						case GDK_BUTTON_RELEASE:
 							gMouse::setControl(NULL);
-							cancel = control->onMouseEvent(control, gEvent_MouseRelease);
+							cancel = CB_control_mouse(control, gEvent_MouseRelease);
 							break;
 					}
 
@@ -614,33 +617,11 @@ __FOUND_WIDGET:
 				}
 			}
 
-			/*if (type == gEvent_MousePress && control->isTopLevel())
-			{
-				gMainWindow *win = ((gMainWindow *)control);
-				if (win->isPopup())
-				{
-					control->getScreenPos(&xc, &yc);
-					xs = (int)event->button.x_root;
-					ys = (int)event->button.y_root;
-					x = xs - xc;
-					y = ys - yc;
-
-					if (x < 0 || y < 0 || x >= win->width() || y >= win->height())
-						win->close();
-				}
-			}
-			else*/ if (type == gEvent_MouseRelease && control->_grab)
+			if (type == gEvent_MouseRelease && control->_grab)
 			{
 				gApplication::exitLoop(control);
 			}
 
-#if GTK_CHECK_VERSION(3, 4, 0)
-			if (gdk_event_triggers_context_menu(event))
-#else
-			if (event->button.button == 3 && event->type == GDK_BUTTON_PRESS)
-#endif
-				menu = true;
-
 			if (!cancel)
 			{
 				if (control->_proxy_for)
@@ -657,7 +638,7 @@ __FOUND_WIDGET:
 				while (control)
 				{
 					//fprintf(stderr, "menu %s D = %d DI = %d\n", control->name(), control->isDesign(), control->isDesignIgnore());
-					if (control->onMouseEvent(control, gEvent_MouseMenu))
+					if (CB_control_mouse(control, gEvent_MouseMenu))
 					{
 						cancel = true;
 						break;
@@ -696,16 +677,18 @@ __FOUND_WIDGET:
 			control = control->ignoreDesign();
 			/*while (control->isDesignIgnore())
 				control = control->parent();*/
-			//fprintf(stderr, "GDK_MOTION_NOTIFY: (%p %s) grab = %p\n", control, control->name(), button_grab);
+			//fprintf(stderr, "GDK_MOTION_NOTIFY: (%p %s) grab = %p state = %d tracking = %d\n", control, control->name(), button_grab, event->motion.state, control->isTracking());
 
 			gApplication::checkHoveredControl(control);
 
 		__MOTION_TRY_PROXY:
 
+			//fprintf(stderr, "--> try (%p %s) / %s\n", control, control->name(), control->_proxy_for ? control->_proxy_for->name() : "-");
+
 			if (!control->isDesign() && !control->isEnabled())
 				goto __HANDLE_EVENT;
 
-			if (control->onMouseEvent && (control->canRaise(control, gEvent_MouseMove) || control->canRaise(control, gEvent_MouseDrag))
+			if ((CB_control_can_raise(control, gEvent_MouseMove) || CB_control_can_raise(control, gEvent_MouseDrag))
 					&& (control->isTracking() || (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))))
 			{
 				control->getScreenPos(&xc, &yc);
@@ -720,7 +703,7 @@ __FOUND_WIDGET:
 
 				//fprintf(stderr, "pressure = %g\n", gMouse::getAxis(GDK_AXIS_PRESSURE));
 
-				cancel = control->onMouseEvent(control, gEvent_MouseMove);
+				cancel = CB_control_mouse(control, gEvent_MouseMove);
 
 				//if (data->acceptDrops() && gDrag::checkThreshold(data, gMouse::x(), gMouse::y(), gMouse::startX(), gMouse::startY()))
 				if (!cancel && (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))
@@ -728,7 +711,7 @@ __FOUND_WIDGET:
 						&& gDrag::checkThreshold(control, gMouse::x(), gMouse::y(), gMouse::startX(), gMouse::startY()))
 				{
 					//fprintf(stderr, "gEvent_MouseDrag: event = %p\n", gApplication::lastEvent());
-					cancel = control->onMouseEvent(control, gEvent_MouseDrag);
+					cancel = CB_control_mouse(control, gEvent_MouseDrag);
 				}
 				gMouse::invalidate();
 
@@ -764,7 +747,7 @@ __FOUND_WIDGET:
 			if (!control->isDesign() && !control->isEnabled())
 				goto __HANDLE_EVENT;
 
-			if (control->onMouseEvent && control->canRaise(control, gEvent_MouseWheel))
+			if (CB_control_can_raise(control, gEvent_MouseWheel))
 			{
 				int dir, dt, ort;
 				
@@ -803,7 +786,7 @@ __FOUND_WIDGET:
 				gMouse::setEvent(event);
 				gMouse::setMouse(x, y, xs, ys, 0, event->scroll.state);
 				gMouse::setWheel(dt, ort);
-				cancel = control->onMouseEvent(control, gEvent_MouseWheel);
+				cancel = CB_control_mouse(control, gEvent_MouseWheel);
 				gMouse::invalidate();
 			}
 
@@ -935,6 +918,7 @@ void (*gApplication::onLeaveEventLoop)();
 bool gApplication::_must_quit = false;
 GdkEvent *gApplication::_event = NULL;
 bool gApplication::_keep_focus = false;
+bool gApplication::_disable_mapping_events = false;
 
 bool gApplication::_fix_breeze = false;
 bool gApplication::_fix_oxygen = false;
@@ -963,6 +947,26 @@ void gApplication::ungrabPopup()
 	}
 }
 
+#ifdef GTK3
+
+bool gApplication::areTooltipsEnabled()
+{
+	return _tooltip_css == NULL;
+}
+
+void gApplication::enableTooltips(bool vl)
+{
+	if (vl == areTooltipsEnabled())
+		return;
+
+	gt_define_style_sheet(&_tooltip_css, NULL);
+
+	if (!vl)
+		gt_define_style_sheet(&_tooltip_css, g_string_new("tooltip { opacity: 0; }"));
+}
+
+#else
+
 bool gApplication::areTooltipsEnabled()
 {
   gboolean enabled;
@@ -983,6 +987,8 @@ void gApplication::enableTooltips(bool vl)
   g_object_set (settings, "gtk-enable-tooltips", enabled, (char *)NULL);
 }
 
+#endif
+
 static void do_nothing()
 {
 }
@@ -1022,8 +1028,6 @@ void gApplication::init(int *argc, char ***argv)
 	
 	appEvents = 0;
 
-	gtk_init(argc, argv);
-
 	#ifdef GTK3
 	_app = gtk_application_new(NULL, G_APPLICATION_FLAGS_NONE);
 	g_object_set(G_OBJECT(_app), "register-session", TRUE, NULL);
@@ -1078,7 +1082,6 @@ void gApplication::exit()
 
 	gKey::exit();
 	gTrayIcon::exit();
-  gDesktop::exit();
   gDialog::exit();
   gFont::exit();
   gt_exit();
@@ -1169,7 +1172,7 @@ GtkWindowGroup *gApplication::enterGroup()
 
 	while (control)
 	{
-		control->emit(SIGNAL(control->onEnterLeave), gEvent_Leave);
+		CB_control_enter_leave(control, gEvent_Leave);
 		control = control->parent();
 	}
 
@@ -1244,11 +1247,13 @@ void gApplication::enterPopup(gMainWindow *owner)
 		_loop_owner = owner;
 
 		(*onEnterEventLoop)();
+		//fprintf(stderr, "event loop <----\n");
 		do
 		{
 			MAIN_do_iteration(false);
 		}
 		while (_loopLevel > l);
+		//fprintf(stderr, "event loop ---->\n");
 		(*onLeaveEventLoop)();
 
 		if (_in_popup == 1)
@@ -1270,8 +1275,8 @@ void gApplication::enterPopup(gMainWindow *owner)
 		}
 		//exitGroup(oldGroup);
 	}
-	else
-		gControl::postDelete();
+	/*else
+		gControl::postDelete();*/
 
 	_in_popup--;
 }
@@ -1308,6 +1313,11 @@ static void post_focus_change(void *)
 {
 	gControl *current, *control, *next;
 
+	#if DEBUG_FOCUS
+	fprintf(stderr, "post_focus_change: %d %d\n", !_focus_change, _doing_focus_change);
+	#endif
+	
+
 	if (!_focus_change || _doing_focus_change)
 		return;
 
@@ -1324,42 +1334,42 @@ static void post_focus_change(void *)
 			break;
 
 		control = gApplication::_old_active_control;
+		//if (control) fprintf(stderr, "check focus out %s\n", control->name());
 		while (control)
 		{
 			next = control->_proxy_for;
 			#if DEBUG_FOCUS
 			fprintf(stderr, "focus out %s\n", control->name());
 			#endif
-			if (control->onFocusEvent)
-				control->onFocusEvent(control, gEvent_FocusOut);
+			CB_control_focus(control, gEvent_FocusOut);
 			control = next;
 		}
-
-		current = gApplication::activeControl();
-		if (current == gApplication::_old_active_control)
-			break;
-
+		
 		gApplication::_old_active_control = current;
 		gMainWindow::setActiveWindow(current);
 
-		control = gApplication::activeControl();
+		control = current; //gApplication::activeControl();
+		//if (control) fprintf(stderr, "check focus in %s\n", control->name());
 		while (control)
 		{
 			next = control->_proxy_for;
 			#if DEBUG_FOCUS
-			fprintf(stderr, "focus in %s / %p\n", control->name(), control->onFocusEvent);
+			fprintf(stderr, "focus in %s\n", control->name());
 			#endif
-			if (control->onFocusEvent)
-				control->onFocusEvent(control, gEvent_FocusIn);
+			CB_control_focus(control, gEvent_FocusIn);
 			control = next;
 		}
 	}
 
-	_doing_focus_change = false;
 	_focus_change = false;
+	_doing_focus_change = false;
+	
+	#if DEBUG_FOCUS
+	fprintf(stderr, "post_focus_change: END\n");
+	#endif
 }
 
-void gApplication::handleFocusNow()
+void gApplication::finishFocus()
 {
 	post_focus_change(NULL);
 }
@@ -1375,6 +1385,16 @@ static void handle_focus_change()
 
 void gApplication::setActiveControl(gControl *control, bool on)
 {
+	if (control->isWindow() && on)
+	{
+		gControl *focus = ((gMainWindow *)control)->getInitialFocus();
+		if (focus != control)
+		{
+			focus->setFocus();
+			control = focus;
+		}
+	}
+	
 	while (!control->canFocus())
 	{
 		control = control->parent();
@@ -1386,9 +1406,12 @@ void gApplication::setActiveControl(gControl *control, bool on)
 		return;
 
 	#if DEBUG_FOCUS
-	fprintf(stderr, "setActiveControl: %s %d\n", control->name(), on);
+	fprintf(stderr, "setActiveControl: %s %s %d / %d\n", GB.GetClassName(control->hFree), control->name(), on, _focus_change);
 	#endif
 	
+	/*if (!::strcmp(GB.GetClassName(control->hFree), "FEditor") && !on)
+		BREAKPOINT();*/
+	
 	if (_active_control && !_focus_change)
 		_previous_control = _active_control;
 
