@@ -24,8 +24,11 @@
 #ifndef __GAPPLICATION_H
 #define __GAPPLICATION_H
 
+#ifndef GTK3
+#include <X11/Xlib.h>
 typedef
 	void (*X11_EVENT_FILTER)(XEvent *);
+#endif
 
 class gControl;
 class gMainWindow;
@@ -39,9 +42,6 @@ public:
 	static void exit();
 	static bool mustQuit() { return _must_quit; }
 
-	static int controlCount();
-
-	static gControl* controlItem(int index);
 	static gControl* controlItem(GtkWidget *wid);
 
   static void setBusy(bool b);
@@ -50,22 +50,17 @@ public:
 	static gControl* activeControl() { return _active_control; }
 	static gControl* previousControl() { return _previous_control; }
 	static void setActiveControl(gControl *control, bool on);
-	static void handleFocusNow();
-	
-	static void suspendEvents(bool vl);
-	static void enableEvents();
-	static bool userEvents();
-	static bool allEvents();
+	static void finishFocus();
 
 	static void enableTooltips(bool vl);
 	static bool areTooltipsEnabled();
 
 	static int dblClickTime();
-	
+
 	static void setDefaultTitle(const char *title);
 	static char *defaultTitle() { return _title; }
 
-	static void setDirty();
+	//static void setDirty();
 	static int loopLevel() { return _loopLevel; }
 	static void enterLoop(void *owner, bool showIt = false, GtkWindow *modal = NULL);
 	static void enterPopup(gMainWindow *owner);
@@ -79,28 +74,37 @@ public:
 	static void updateLastEventTime();
 
 	static bool (*onKeyEvent)(int type);
-	
+
 	static int getScrollbarSize();
+	static int getScrollbarBigSize();
 	static int getScrollbarSpacing();
 	static int getFrameWidth();
 	static int getInnerWidth();
 	static void getBoxFrame(int *w, int *h);
 	static char *getStyleName();
-	
+
 	static void grabPopup();
 	static void ungrabPopup();
-	
+
 	static void setMainWindow(gMainWindow *win);
 	static gMainWindow *mainWindow() { return _main_window; }
-	
+
 	static void checkHoveredControl(gControl *control);
 
+	#ifndef GTK3
 	static void setEventFilter(X11_EVENT_FILTER filter);
+	#endif
 
 	static void setButtonGrab(gControl *grab) { _button_grab = grab; }
 	
-	static bool fix_breeze;
-	static bool fix_oxygen;
+	static void onThemeChange();
+	
+	static void forEachControl(void (*cb)(gControl *), bool (*filter)(gControl *) = NULL);
+	
+	static bool _fix_breeze;
+	static bool _fix_oxygen;
+	static int _scrollbar_size;
+	static int _scrollbar_big_size;
 
 	//"Private"
 	static bool _init;
@@ -114,7 +118,6 @@ public:
 	static void *_loop_owner;
 	static GtkWindowGroup *_group;
 	static GtkWindowGroup *currentGroup();
-	//static void dispatchEnterLeave(gControl *enter);
 	static gControl *_enter;
 	static gControl *_leave;
 	static gControl *_ignore_until_next_enter;
@@ -131,6 +134,8 @@ public:
 	static bool _fix_printer_dialog;
 	static void (*onEnterEventLoop)();
 	static void (*onLeaveEventLoop)();
+	static bool _keep_focus;
+	static bool _disable_mapping_events;
 };
 
 #endif
