@@ -138,11 +138,6 @@ public:
 	void enableArrangement();
 	bool isArrangementEnabled() const { return _no_arrangement == 0; }
 	
-//"Signals"
-	void (*onArrange)(gContainer *sender);
-	void (*onBeforeArrange)(gContainer *sender);
-	//void (*onInsert)(gContainer *sender, gControl *child);
-
 //"Private"
 	GtkWidget *radiogroup;
 	GPtrArray *_children;
@@ -154,13 +149,15 @@ public:
 	virtual void reparent(gContainer *newpr, int x, int y);
 	void hideHiddenChildren();
 	virtual GtkWidget *getContainer();
-
+	
 	void setShown(bool v) { _shown = v; }
 	bool isShown() const { return _shown; }
 	void arrangeLater();
 	void resetArrangeLater();
 	
-	virtual void connectBorder();
+	virtual void borderSignals();
+	
+	virtual void updateDirection();
 	
 	static void postArrange();
 	
@@ -179,4 +176,11 @@ private:
 	unsigned char _no_arrangement;
 };
 
+// Callbacks
+//"Signals"
+void CB_container_arrange(gContainer *sender);
+void CB_container_before_arrange(gContainer *sender);
+
+
+
 #endif
