@@ -771,15 +771,6 @@ bool gControl::resize(int w, int h, bool no_decide)
 	}
 	else
 	{
-		/*if (frame && widget != border)
-		{
-			int fw = getFrameWidth() * 2;
-			if (w < fw || h < fw)
-				gtk_widget_hide(widget);
-			else
-				gtk_widget_show(widget);
-		}*/
-
 		//g_debug("resize: %p %s: %d %d", this, name(), w, h);
 		_dirty_size = true;
 
@@ -1541,15 +1532,18 @@ Drag & Drop
 
 void gControl::setAcceptDrops(bool vl)
 {
+	GtkWidget *w;
+
 	if (vl == _accept_drops)
 		return;
 
 	_accept_drops = vl;
 
+	w = _scroll ? widget : border;
 	if (vl)
-		gtk_drag_dest_set(border, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+		gtk_drag_dest_set(w, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	else
-		gtk_drag_dest_unset(border);
+		gtk_drag_dest_unset(w);
 }
 
 /*********************************************************************
@@ -2634,6 +2628,9 @@ bool gControl::setProxy(gControl *proxy)
 
 	//fprintf(stderr, "proxy: (%p %s) -> (%p %s)\n", this, name(), proxy, proxy ? proxy->name() : "NULL");
 	
+	if (proxy && proxy->_proxy_for)
+		proxy->_proxy_for->_proxy = NULL;
+
 	if (_proxy)
 		_proxy->_proxy_for = NULL;
 
