@@ -133,8 +133,17 @@ public:
 	void setTracking(bool vl);
 	
 	bool isNoTabFocus() const;
+	bool isNoTabFocusRec() const;
 	void setNoTabFocus(bool v);
+	
+	void setDirection(int v);
+	int direction() const { return _direction; }
+	
+	void setInverted(bool v);
+	bool isInverted() const { return _inverted; }
 
+	bool isRightToLeft() const;
+	
 	gColor background() const { return _bg; }
 	gColor foreground() const { return _fg; }
 	virtual void setBackground(gColor color = COLOR_DEFAULT);
@@ -224,20 +233,6 @@ public:
 	void emit(void *signal, intptr_t arg);
 	void emit(void *signal, char *arg) { emit(signal, (intptr_t)arg); }
 
-// "Signals"
-	bool (*canRaise)(gControl *sender, int type);
-	void (*onFinish)(gControl *sender); // Special
-	void (*onFocusEvent)(gControl *sender, int type);
-	bool (*onKeyEvent)(gControl *sender, int type);
-	bool (*onMouseEvent)(gControl *sender, int type);
-	void (*onEnterLeave)(gControl *sender, int type);
-	bool (*onDrag)(gControl *sender);
-	bool (*onDragMove)(gControl *sender);
-	bool (*onDrop)(gControl *sender);
-	void (*onDragLeave)(gControl *sender);
-	//void (*onMove)(gControl *sender);
-	//void (*onResize)(gControl *sender);
-
 // "Private"
 	gint bufW,bufH,bufX,bufY;
 	int _min_w, _min_h;
@@ -304,9 +299,10 @@ public:
 	unsigned _has_native_popup : 1;        // I have a native popup menu
 	unsigned _eat_return_key : 1;          // If the control eats the return key
 	unsigned _minimum_size_set : 1;        // If minimum size has been computed
-	unsigned _hidden_temp : 1;             // Temporarily hidden (must not raise watchers)
 	unsigned _allow_show : 1;              // Allowed to be visible (after the first resize)
 	
+	unsigned _direction : 2;               // Text direction
+	
 #ifdef GTK3
 	unsigned _style_dirty : 1;             // If the style must be refreshed
 	unsigned _no_style_without_child : 1;  // For containers that do not need a css if they have no children
@@ -315,7 +311,7 @@ public:
 	
   void removeParent() { pr = NULL; }
 	void initSignals();
-	void borderSignals();
+	virtual void borderSignals();
 	void widgetSignals();
 	void connectParent();
 	void setParent(gContainer *parent) { pr = parent; }
@@ -325,6 +321,7 @@ public:
 	void registerControl();
 	void updateGeometry(bool force = false);
 	bool mustUpdateCursor() { return mouse() != -1 || have_cursor || !parent(); }
+	void updateEventMask();
 	
 	bool hasInputMethod() { return _has_input_method; }
 	virtual GtkIMContext *getInputMethod();
@@ -349,7 +346,6 @@ public:
 	
 	void createBorder(GtkWidget *new_border, bool keep_widget = false);
 	void createWidget();
-	virtual void connectBorder();
 	
 	int minimumWidth() const { return _min_w; }
 	int minimumHeight() const { return _min_h; }
@@ -364,12 +360,11 @@ public:
 	virtual void onLeaveEvent();
 #endif
 	
-	virtual bool setInverted(bool v);
-	bool isInverted() const { return _inverted; }
-	
 	void hideButKeepFocus();
 	void showButKeepFocus();
-	bool isTempHidden() const { return _hidden_temp; }
+	
+	int actualDirection() const;
+	virtual void updateDirection();
 	
 	static void postDelete();
 
@@ -384,4 +379,18 @@ private:
 
 #define SIGNAL(_signal) ((void *)_signal)
 
+// Callbacks
+
+// "Signals"
+bool CB_control_can_raise(gControl *sender, int type);
+void CB_control_finish(gControl *sender);
+void CB_control_focus(gControl *sender, int type);
+bool CB_control_key(gControl *sender, int type);
+bool CB_control_mouse(gControl *sender, int type);
+void CB_control_enter_leave(gControl *sender, int type);
+bool CB_control_drag(gControl *sender);
+bool CB_control_drag_move(gControl *sender);
+bool CB_control_drop(gControl *sender);
+void CB_control_drag_leave(gControl *sender);
+
 #endif
