@@ -93,13 +93,16 @@ static gboolean cb_frame(GtkWidget *widget,GdkEventWindowState *event,gMainWindo
 	}
 
 	if (event->changed_mask & (GDK_WINDOW_STATE_ICONIFIED | GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN | GDK_WINDOW_STATE_STICKY | GDK_WINDOW_STATE_ABOVE | GDK_WINDOW_STATE_BELOW))
-		data->emit(SIGNAL(data->onState));
+		CB_window_state(data);
 
 	return false;
 }
 
 static gboolean cb_show(GtkWidget *widget, gMainWindow *data)
 {
+	if (gApplication::_disable_mapping_events)
+		return false;
+	
 	if (data->_grab_on_show)
 	{
 		data->_grab_on_show = FALSE;
@@ -115,7 +118,7 @@ static gboolean cb_show(GtkWidget *widget, gMainWindow *data)
 		fprintf(stderr, "cb_show\n");
 		#endif
 		data->emitResize();
-		data->emit(SIGNAL(data->onShow));
+		CB_window_show(data);
 		data->_not_spontaneous = false;
 	}
 	return false;
@@ -123,25 +126,32 @@ static gboolean cb_show(GtkWidget *widget, gMainWindow *data)
 
 static gboolean cb_map(GtkWidget *widget, GdkEvent *event, gMainWindow *data)
 {
+	if (gApplication::_disable_mapping_events)
+		return false;
+	
 	data->_unmap = false;
 	return cb_show(widget, data);
 }
 
 static gboolean cb_hide(GtkWidget *widget, gMainWindow *data)
 {
+	if (gApplication::_disable_mapping_events)
+		return false;
+	
 	if (!data->_unmap)
 	{
-		data->emit(SIGNAL(data->onHide));
+		CB_window_hide(data);
 		data->_not_spontaneous = false;
 	}
 
 	return false;
-	//if (data == gDesktop::activeWindow())
-	//	gMainWindow::setActiveWindow(NULL);
 }
 
 static gboolean cb_unmap(GtkWidget *widget, GdkEvent *event, gMainWindow *data)
 {
+	if (gApplication::_disable_mapping_events)
+		return false;
+	
 	bool ret = cb_hide(widget, data);
 	data->_unmap = true;
 	return ret;
@@ -224,7 +234,7 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 	{
 		data->bufX = x;
 		data->bufY = y;
-		if (data->onMove) data->onMove(data);
+		CB_window_move(data);
 	}
 	
 	#ifdef DEBUG_RESIZE
@@ -235,6 +245,10 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 
 	data->bufW = event->width - data->_csd_w;
 	data->bufH = event->height - data->_csd_h;
+	
+	#ifdef DEBUG_RESIZE
+	fprintf(stderr, "-> %d %d\n", data->bufW, data->bufH);
+	#endif
 		
 	data->emitResize();
 
@@ -319,6 +333,7 @@ static gboolean my_key_press_event(GtkWidget *widget, GdkEventKey *event)
 	GtkWidget *focus;
 
 	focus = gtk_window_get_focus(window);
+	//fprintf(stderr, "focus %p = %p\n", window, focus);
 	if (focus && gtk_widget_get_realized(focus))
 	{
 		if (GTK_IS_ENTRY(focus) || GTK_IS_TEXT_VIEW(focus))
@@ -335,7 +350,7 @@ static gboolean my_key_press_event(GtkWidget *widget, GdkEventKey *event)
 	if (handled)
 		return TRUE;
 
-	if (!propagated && gtk_widget_get_realized(focus))
+	if (!propagated && focus && gtk_widget_get_realized(focus))
 	{
 		handled = gtk_window_propagate_key_event(window, event);
 		if (handled)
@@ -388,12 +403,13 @@ void gMainWindow::initialize()
 	menuBar = NULL;
 	_icon = NULL;
 	_picture = NULL;
-	focus = 0;
+	_initial_focus = NULL;
+	_save_focus = NULL;
 	_title = NULL;
-	_current = NULL;
 	_resize_last_w = _resize_last_h = -1;
 	_min_w = _min_h = _default_min_w = _default_min_h = 0;
 	_csd_w  = _csd_h = -1;
+	_previous = NULL;
 
 	_opened = false;
 	_sticky = false;
@@ -424,17 +440,8 @@ void gMainWindow::initialize()
 	_is_window = true;
 	_no_background = true;
 	_frame_init = false;
+	_set_focus = false;
 	
-	onOpen = NULL;
-	onShow = NULL;
-	onHide = NULL;
-	onMove = NULL;
-	onResize = NULL;
-	onActivate = NULL;
-	onDeactivate = NULL;
-	onState = NULL;
-	onFontChange = NULL;
-
 	accel = gtk_accel_group_new();
 }
 
@@ -480,7 +487,6 @@ void gMainWindow::initWindow()
 
 	have_cursor = true; //parent() == 0 && !_xembed;
 	setCanFocus(true);
-	setNoTabFocus(true);
 }
 
 
@@ -560,20 +566,20 @@ gMainWindow::gMainWindow(gContainer *par) : gContainer(par)
 	
 	gtk_widget_show(frame);
 	gtk_widget_show(widget);
+	
+	setVisibility(false);
 }
 
 gMainWindow::~gMainWindow()
 {
-	//fprintf(stderr, "delete window %p %s\n", this, name());
-	
 	if (!border)
 		return;
 
-	gApplication::handleFocusNow();
+	gApplication::finishFocus();
 
 	if (_opened)
 	{
-		emit(SIGNAL(onClose));
+		CB_window_close(this);
 		_opened = false;
 		if (GTK_IS_WINDOW(border) && isModal())
 			gApplication::exitLoop(this);
@@ -747,7 +753,7 @@ bool gMainWindow::emitOpen()
 
 	gtk_widget_realize(border);
 
-	emit(SIGNAL(onOpen));
+	CB_window_open(this);
 	if (_closed)
 	{
 		_opened = false;
@@ -755,7 +761,7 @@ bool gMainWindow::emitOpen()
 	}
 
 	//fprintf(stderr, "emit Move & Resize: %p\n", this);
-	emit(SIGNAL(onMove));
+	CB_window_move(this);
 	#ifdef DEBUG_RESIZE
 	fprintf(stderr, "cb_open\n");
 	#endif
@@ -789,9 +795,6 @@ void gMainWindow::setTransientFor()
 {
 	gMainWindow *parent = _current;
 
-	if (!parent)
-		parent = gApplication::mainWindow();
-		
 	if (!parent)
 		parent = _active;
 
@@ -818,7 +821,10 @@ void gMainWindow::setVisible(bool vl)
 	{
 		gContainer::setVisible(vl);
 		if (vl)
-			setActiveWindow(this);
+		{
+			_hidden = false;
+			//setActiveWindow(this);
+		}
 		return;
 	}
 
@@ -895,34 +901,9 @@ void gMainWindow::setVisible(bool vl)
 		}
 
 		drawMask();
+		
+		_set_focus = true;
 
-		if (focus)
-		{
-			//fprintf(stderr, "focus = %s\n", focus->name());
-			focus->setFocus();
-			focus = NULL;
-		}
-		else
-		{
-			gControl *ctrl = this;
-			
-			for(;;)
-			{
-				ctrl = ctrl->nextFocus();
-				if (!ctrl)
-					break;
-				
-				if (ctrl->isReallyVisible() && ctrl->isEnabled() && ctrl->canFocus())
-				{
-					ctrl->setFocus();
-					break;
-				}
-				
-				if (ctrl == this)
-					break;
-			}
-		}
-			
 		if (isSkipTaskBar())
 			_activate = true;
 
@@ -935,7 +916,7 @@ void gMainWindow::setVisible(bool vl)
 	else
 	{
 		if (this == _active)
-			focus = gApplication::activeControl();
+			_initial_focus = gApplication::activeControl();
 
 		_not_spontaneous = isVisible();
 		gContainer::setVisible(false);
@@ -1022,11 +1003,10 @@ bool gMainWindow::isModal() const
 
 void gMainWindow::showModal()
 {
-  gMainWindow *save;
-
 	if (!isTopLevel()) return;
-	if (isModal()) return;
+	if (isModal() || isPopup()) return;
 
+	gApplication::finishFocus();
 	gMouse::finishEvent();
 	
 	//show();
@@ -1035,35 +1015,43 @@ void gMainWindow::showModal()
 	gtk_window_set_modal(GTK_WINDOW(border), true);
 	setTransientFor();
 
-	save = _current;
+	_save_focus = gApplication::activeControl();
+	_previous = _current;
 	_current = this;
 
 	center();
 	show();
-	gtk_grab_add(border);
+	//gtk_grab_add(border);
 	gApplication::enterLoop(this);
 
-	_current = save;
+	_current = _previous;
+	_previous = NULL;
 
-	gtk_grab_remove(border);
+	//gtk_grab_remove(border);
 	gtk_window_set_modal(GTK_WINDOW(border), false);
 
 	if (!_persistent)
-		destroyNow();
+		destroy();
 	else
 		hide();
+	
+	if (_save_focus)
+	{
+		gApplication::setActiveControl(_save_focus, true);
+		_save_focus = NULL;
+	}
 }
 
 void gMainWindow::showPopup(int x, int y)
 {
-  gMainWindow *save;
 	bool has_border;
 	int oldx, oldy;
 	GdkWindowTypeHint type;
 
 	if (!isTopLevel()) return;
-	if (isModal()) return;
+	if (isModal() || isPopup()) return;
 
+	gApplication::finishFocus();
 	gMouse::finishEvent();
 
 	//gtk_widget_unrealize(border);
@@ -1084,23 +1072,23 @@ void gMainWindow::showPopup(int x, int y)
 	
 	setTransientFor();
 
+	_save_focus = gApplication::activeControl();
+	_previous = _current;
+	_current = this;
+
 	gtk_window_resize(GTK_WINDOW(border), bufW, bufH);
   move(x, y);
 	//raise();
 	setFocus();
-
-	save = _current;
-	_current = this;
-
+	
 	gApplication::enterPopup(this);
 
-	_current = save;
+	_current = _previous;
+	_previous = NULL;
 	_popup = false;
 
 	if (!_persistent)
-	{
-		destroyNow();
-	}
+		destroy();
 	else
 	{
 		hide();
@@ -1110,6 +1098,12 @@ void gMainWindow::showPopup(int x, int y)
 
 		move(oldx, oldy);
 	}
+	
+	if (_save_focus)
+	{
+		gApplication::setActiveControl(_save_focus, true);
+		_save_focus = NULL;
+	}
 }
 
 void gMainWindow::showActivate()
@@ -1365,7 +1359,7 @@ bool gMainWindow::doClose(bool destroying)
 		if (_opened)
 		{
 			_closing = true;
-			_closed = !onClose(this);
+			_closed = !CB_window_close(this);
 			_closing = false;
 			_opened = !_closed;
 		}
@@ -1388,7 +1382,7 @@ bool gMainWindow::doClose(bool destroying)
 				return true;
 
 			_closing = true;
-			_closed = !onClose(this);
+			_closed = !CB_window_close(this);
 			_closing = false;
 			_opened = !_closed;
 
@@ -1441,6 +1435,7 @@ void gMainWindow::createWindow(GtkWidget *new_border)
 {
 	gt_widget_reparent(frame, new_border);
 	createBorder(new_border);
+	updateEventMask();
 	registerControl();
 }
 
@@ -1448,15 +1443,18 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 {
 	int w, h;
 	gColor fg, bg;
+	bool was_visible;
 
 	if (_xembed)
 		return;
 
 	bg = background();
 	fg = foreground();
+	was_visible = isVisible();
 
 	if (isTopLevel() && newpr)
 	{
+		windows = g_list_remove(windows, (gpointer)this);
 		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
 
 		createWindow(gtk_event_box_new());
@@ -1478,21 +1476,28 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 
 		gtk_widget_set_size_request(border, width(), height());
 
+		if (was_visible)
+			gtk_widget_show(border);
 		// Hidden children are incorrectly shown. Fix that!
 		hideHiddenChildren();
+		
+		if (!isIgnore())
+			newpr->performArrange();
 	}
 	else if ((!isTopLevel() && !newpr)
 	         || (isTopLevel() && isPopup()))
 	{
+		windows = g_list_append(windows, (gpointer)this);
 		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
 		// TODO: test that
 		
 		createWindow(gtk_window_new(GTK_WINDOW_TOPLEVEL));
-
+		
 		if (parent())
 		{
 			parent()->remove(this);
-			parent()->arrange();
+			if (!isIgnore())
+				parent()->performArrange();
 			setParent(NULL);
 		}
 
@@ -1511,11 +1516,13 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		gtk_widget_set_size_request(border, 1, 1);
 		resize(w, h);
 
-		hideHiddenChildren();
-		
 		gtk_widget_set_sensitive(frame, FALSE);
 		gtk_widget_set_sensitive(frame, TRUE);
 
+		if (was_visible)
+			present();
+		hideHiddenChildren();
+		
 		_popup = false; //type == GTK_WINDOW_POPUP;
 	}
 	else
@@ -1657,21 +1664,7 @@ int gMainWindow::clientHeight()
 
 void gMainWindow::setActiveWindow(gControl *control)
 {
-	gMainWindow *window = control ? control->window() : NULL;
-	gMainWindow *old = _active;
-
-	if (window == _active)
-		return;
-
-	_active = window;
-
-	//fprintf(stderr, "setActiveWindow: %p %s\n", _active, _active ? _active->name() : "");
-
-	if (old)
-		old->emit(SIGNAL(old->onDeactivate));
-
-	if (window)
-		window->emit(SIGNAL(window->onActivate));
+	_active = CB_window_activate(control);
 }
 
 #ifdef GDK_WINDOWING_X11
@@ -1767,17 +1760,17 @@ void gMainWindow::configure()
 bool gMainWindow::setMenuBarVisible(bool v)
 {
 	if (_showMenuBar == v)
-		return TRUE;
+		return true;
 
 	_showMenuBar = v;
 
 	if (!menuBar)
-		return TRUE;
+		return true;
 
 	configure();
 	performArrange();
 
-	return FALSE;
+	return false;
 }
 
 bool gMainWindow::isMenuBarVisible()
@@ -1790,7 +1783,7 @@ void gMainWindow::updateFont()
 {
 	gContainer::updateFont();
 	gMenu::updateFont(this);
-	emit(SIGNAL(onFontChange));
+	CB_window_font(this);
 }
 
 void gMainWindow::checkMenuBar()
@@ -1895,7 +1888,7 @@ void gMainWindow::emitResize()
 	_resize_last_h = bufH;
 	configure();
 	performArrange();
-	emit(SIGNAL(onResize));
+	CB_window_resize(this);
 }
 
 static void emit_resize_later(gMainWindow *window)
@@ -2055,17 +2048,18 @@ void gMainWindow::calcCsdSize()
 	}
 		
 	gtk_widget_get_allocation(border, &ba);
-	gtk_widget_get_allocation(frame, &wa);
-	
-	if (wa.width == 1 && wa.height == 1)
+	if (ba.width <= 1 && ba.height <= 1)
 		return;
 
-	//fprintf(stderr, "border: %d %d layout: %d %d\n", ba.width, ba.height, wa.width, wa.height);
+	gtk_widget_get_allocation(frame, &wa);
+	if (wa.width <= 1 && wa.height <= 1)
+		return;
 	
 	_csd_w = ba.width - wa.width;
 	_csd_h = ba.height - wa.height;
 	#ifdef DEBUG_RESIZE
-	fprintf(stderr, "calcCsdSize: %s: csd = %d %d\n", name(), _csd_w, _csd_h);
+	fprintf(stderr, "calcCsdSize: border: %d %d layout: %d %d\n", ba.width, ba.height, wa.width, wa.height);
+	fprintf(stderr, "calcCsdSize: --> %s: csd = %d %d\n", name(), _csd_w, _csd_h);
 	#endif
 	
 	if (!isResizable())
@@ -2097,3 +2091,48 @@ void gMainWindow::getCustomMinimumSize(int *w, int *h) const
 	*h = _min_h;
 }
 
+gControl *gMainWindow::getInitialFocus()
+{
+	gControl *ctrl;
+	
+	if (!_set_focus)
+		return this;
+	
+	_set_focus = false;
+	
+	if (_initial_focus)
+	{
+		ctrl = _initial_focus;
+		_initial_focus = NULL;
+		//fprintf(stderr, "focus = %p %s\n", focus->border, focus->name());
+		//focus->setFocus();
+		//fprintf(stderr, "focus of window %p -> %p\n", border, gtk_window_get_focus(GTK_WINDOW(border)));
+		//focus = NULL;
+	}
+	else
+	{
+		ctrl = this;
+		
+		for(;;)
+		{
+			ctrl = ctrl->nextFocus();
+			if (!ctrl)
+				break;
+			
+			if (ctrl->isReallyVisible() && ctrl->isEnabled() && !ctrl->isWindow() && ctrl->canFocus())
+				break;
+			
+			if (ctrl == this)
+				break;
+		}
+	}
+	
+	return ctrl ? ctrl : this;
+}
+
+#ifdef GTK3
+GtkWidget *gMainWindow::getStyleSheetWidget()
+{
+	return frame;
+}
+#endif
