@@ -74,7 +74,7 @@ static gboolean raise_change(gTextBox *data)
 {
 	if (data->_changed)
 	{
-		data->emit(SIGNAL(data->onChange));
+		CB_textbox_change(data);
 		data->_changed = false;
 	}
 	return FALSE;
@@ -95,7 +95,7 @@ static void cb_change_insert(GtkEditable *editable, gchar *new_text, gint new_te
 {
 	data->_changed = false;
 	gtk_editable_set_position(editable, *position);
-	data->emit(SIGNAL(data->onChange));
+	CB_textbox_change(data);
 	*position = gtk_editable_get_position(editable);
 }
 
@@ -108,9 +108,18 @@ static void cb_change_delete(GtkEditable *editable, gint start_pos, gint end_pos
 	}
 }
 
-static void cb_activate(GtkEntry *editable,gTextBox *data)
+static void cb_activate(GtkEntry *editable, gTextBox *data)
 {
-	data->emit(SIGNAL(data->onActivate));
+	CB_textbox_activate(data);
+}
+
+static void cb_cursor(GtkWidget *entry, GParamSpec *param, gTextBox *data)
+{
+	if (data->_last_position != data->position())
+	{
+		data->_last_position = data->position();
+		CB_textbox_cursor(data);
+	}
 }
 
 gTextBox::gTextBox(gContainer *parent, bool combo) : gControl(parent)
@@ -122,6 +131,7 @@ gTextBox::gTextBox(gContainer *parent, bool combo) : gControl(parent)
 	_has_border = true;
 	_has_native_popup = true;
 	_text_area_visible = true;
+	_last_position = 0;
 
 	if (!combo)
 	{
@@ -135,9 +145,6 @@ gTextBox::gTextBox(gContainer *parent, bool combo) : gControl(parent)
 	}
 	else
 		entry = NULL;
-	
-	onChange = NULL;
-	onActivate = NULL;
 }
 
 gTextBox::~gTextBox()
@@ -164,6 +171,7 @@ void gTextBox::initEntry()
 	g_signal_connect_after(G_OBJECT(entry), "insert-text", G_CALLBACK(cb_change_insert), (gpointer)this);
 	g_signal_connect_after(G_OBJECT(entry), "delete-text", G_CALLBACK(cb_change_delete), (gpointer)this);
 	g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(cb_activate), (gpointer)this);
+	g_signal_connect(G_OBJECT(entry), "notify::cursor-position", G_CALLBACK(cb_cursor), (gpointer)this);
 	//g_signal_connect(getInputMethod(), "commit", G_CALLBACK(cb_im_commit), (gpointer)this);
 }
 
@@ -174,16 +182,21 @@ char* gTextBox::text()
 
 void gTextBox::setText(const char *vl)
 {
+	int save;
+	
 	if (!vl) vl = "";
 	
 	if (!entry || !strcmp(vl, text()))
 		return;
-		
+	
+	save = _last_position;
 	lock();
 	gtk_entry_set_text(GTK_ENTRY(entry), vl);
 	gtk_editable_set_position(GTK_EDITABLE(entry), -1);
 	unlock();
-	emit(SIGNAL(onChange));
+	CB_textbox_change(this);
+	_last_position = save;
+	cb_cursor(entry, NULL, this);
 }
 
 #ifdef GTK3
@@ -456,8 +469,9 @@ void gTextBox::clear()
 
 void gTextBox::setMinimumSize()
 {
-	_min_h = font()->height() + (hasBorder() ? 4 : 0);
-	_min_w = _min_h;
+	int h = font()->height();
+	_min_h = h + (hasBorder() ? 4 : 0);
+	_min_w = h / 2 + (hasBorder() ? 4 : 0);
 }
 
 
@@ -530,7 +544,10 @@ void gTextBox::onEnterEvent()
 		return;
 	
 	if (_text_area_visible)
+	{
+		//fprintf(stderr, "gTextBox::onEnterEvent: show\n");
 		gdk_window_show(TEXT_AREA(entry));
+	}
 }
 
 void gTextBox::onLeaveEvent()
@@ -540,7 +557,10 @@ void gTextBox::onLeaveEvent()
 	
 	_text_area_visible = !hasFocus() && gdk_window_is_visible(TEXT_AREA(entry));
 	if (_text_area_visible)
+	{
+		//fprintf(stderr, "gTextBox::onEnterEvent: hide\n");
 		gdk_window_hide(TEXT_AREA(entry));
+	}
 }
 #endif
 
