@@ -227,6 +227,13 @@ void gt_to_css_color(char *css, gColor color)
 	}
 }
 
+void gt_add_css_color(char **pcss, gColor color)
+{
+	char buffer[64];
+	gt_to_css_color(buffer, color);
+	g_stradd(pcss, buffer);
+}
+
 #endif
 
 gColor gt_frgba_to_color(double r, double g, double b, double a)
@@ -1638,7 +1645,7 @@ static void set_layout_from_font(PangoLayout *layout, gFont *font, bool add, int
 	PangoAttribute *attr;
 	bool copy = false;
 	
-	desc = pango_context_get_font_description(font->ct);
+	desc = font->desc();
 
 	/*if ((dpi && dpi != gDesktop::resolution()))
 	{
@@ -1704,7 +1711,7 @@ void gt_add_layout_from_font(PangoLayout *layout, gFont *font, int dpi)
 	set_layout_from_font(layout, font, true, dpi);
 }
 
-void gt_layout_alignment(PangoLayout *layout, float w, float h, float *tw, float *th, int align, float *offX, float *offY)
+void gt_layout_alignment(PangoLayout *layout, const char *text, int len, float w, float h, float *tw, float *th, int align, float *offX, float *offY)
 {
 	int ptw, pth;
 	gt_layout_get_extents(layout, &ptw, &pth, FALSE);
@@ -1714,19 +1721,24 @@ void gt_layout_alignment(PangoLayout *layout, float w, float h, float *tw, float
 	if (w < 0) w = *tw;
 	if (h < 0) h = *th;
 
-	switch (align)
+	if (ALIGN_IS_NORMAL(align))
 	{
-		case ALIGN_BOTTOM_NORMAL:
-			align = gDesktop::rightToLeft() ? ALIGN_BOTTOM_RIGHT : ALIGN_BOTTOM_LEFT;
-			break;
+		PangoDirection dir = pango_find_base_dir(text, len);
 		
-		case ALIGN_NORMAL:
-			align = gDesktop::rightToLeft() ? ALIGN_RIGHT : ALIGN_LEFT;
-			break;
+		switch (align)
+		{
+			case ALIGN_BOTTOM_NORMAL:
+				align = dir == PANGO_DIRECTION_RTL ? ALIGN_BOTTOM_RIGHT : ALIGN_BOTTOM_LEFT;
+				break;
 			
-		case ALIGN_TOP_NORMAL:
-			align = gDesktop::rightToLeft() ? ALIGN_TOP_RIGHT : ALIGN_TOP_LEFT;
-			break;
+			case ALIGN_NORMAL:
+				align = dir == PANGO_DIRECTION_RTL ? ALIGN_RIGHT : ALIGN_LEFT;
+				break;
+				
+			case ALIGN_TOP_NORMAL:
+				align = dir == PANGO_DIRECTION_RTL ? ALIGN_TOP_RIGHT : ALIGN_TOP_LEFT;
+				break;
+		}
 	}
 	
 	switch (align)
@@ -2569,12 +2581,3 @@ void gt_layout_get_extents(PangoLayout *layout, int *w, int *h, bool pixels)
 	
 }
 
-void gt_widget_set_inverted(GtkWidget *widget, bool inverted)
-{
-	GtkTextDirection dir = gtk_widget_get_default_direction();
-	
-	if (inverted)
-		gtk_widget_set_direction(widget, dir == GTK_TEXT_DIR_LTR ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR);
-	else
-		gtk_widget_set_direction(widget, dir);
-}
