@@ -122,7 +122,7 @@ gPicture *gt_grab_window(GdkWindow *win, int x = 0, int y = 0, int w = 0, int h
 
 // Compute the alignment of a PangoLayout
 
-void gt_layout_alignment(PangoLayout *layout, float w, float h, float *tw, float *th, int align, float *offX, float *offY);
+void gt_layout_alignment(PangoLayout *layout, const char *text, int len, float w, float h, float *tw, float *th, int align, float *offX, float *offY);
 
 // Compute the extents of a layout
 
@@ -180,6 +180,7 @@ gColor gt_frgba_to_color(double r, double g, double b, double a);
 void gt_from_color(gColor color, GdkRGBA *rgba);
 gColor gt_to_color(GdkRGBA *rgba);
 void gt_to_css_color(char *css, gColor color);
+void gt_add_css_color(char **pcss, gColor color);
 #endif
 
 // Draw a control border
@@ -240,6 +241,4 @@ void gt_widget_update_css(GtkWidget *widget, gFont *font, gColor bg, gColor fg);
 void gt_define_style_sheet(GtkStyleProvider **provider, GString *css);
 #endif
 
-void gt_widget_set_inverted(GtkWidget *widget, bool inverted);
-
 #endif
