@@ -156,7 +156,7 @@ static int to_gambas_event(int type)
 	}
 }
 
-static bool can_raise(gControl *sender, int type)
+bool CB_control_can_raise(gControl *sender, int type)
 {
 	CWIDGET *ob = GetObject(sender);
 	if (!ob)
@@ -169,7 +169,7 @@ static bool can_raise(gControl *sender, int type)
 	return GB.CanRaise(ob, type);
 }
 
-static bool cb_mouse(gControl *sender, int type)
+bool CB_control_mouse(gControl *sender, int type)
 {
 	CWIDGET *ob = GetObject(sender);
 	bool ret = false;
@@ -225,22 +225,22 @@ static bool cb_mouse(gControl *sender, int type)
 	return ret;
 }
 
-static bool cb_key(gControl *sender, int type)
+bool CB_control_key(gControl *sender, int type)
 {
 	return GB.Raise(GetObject(sender), to_gambas_event(type), 0);
 }
 
-static void cb_enter_leave(gControl *sender, int type)
+void CB_control_enter_leave(gControl *sender, int type)
 {
 	GB.Raise(GetObject(sender), to_gambas_event(type), 0);
 }
 
-static void cb_focus(gControl *sender, int type)
+void CB_control_focus(gControl *sender, int type)
 {
 	GB.Raise(GetObject(sender), to_gambas_event(type), 0);
 }
 
-static bool cb_drag(gControl *sender)
+bool CB_control_drag(gControl *sender)
 {
 	CWIDGET *_object = GetObject(sender);
 
@@ -258,14 +258,14 @@ static bool cb_drag(gControl *sender)
 	return GB.Raise(THIS, EVENT_Drag, 0);
 }
 
-static void cb_drag_leave(gControl *sender)
+void CB_control_drag_leave(gControl *sender)
 {
 	CWIDGET *_object = GetObject(sender);
 
 	GB.Raise(THIS, EVENT_DragLeave, 0);
 }
 
-static bool cb_drag_move(gControl *sender)
+bool CB_control_drag_move(gControl *sender)
 {
 	CWIDGET *_object = GetObject(sender);
 
@@ -278,7 +278,7 @@ static bool cb_drag_move(gControl *sender)
 	return GB.Raise(THIS, EVENT_DragMove, 0);
 }
 
-static bool cb_drop(gControl *sender)
+bool CB_control_drop(gControl *sender)
 {
 	CWIDGET *_object = GetObject(sender);
 
@@ -292,7 +292,7 @@ static bool cb_drop(gControl *sender)
 	return true;
 }
 
-void DeleteControl(gControl *control)
+void CB_control_finish(gControl *control)
 {
 	CWIDGET *widget = (CWIDGET*)control->hFree;
 
@@ -346,39 +346,21 @@ void InitControl(gControl *control, CWIDGET *widget)
 	else
 		control->setName(name);
 
-	control->onFinish = DeleteControl;
-	control->onMouseEvent = cb_mouse;
-	control->onKeyEvent = cb_key;
-	control->onFocusEvent = cb_focus;
-	control->onDrag = cb_drag;
-	control->onDragLeave = cb_drag_leave;
-	control->onDragMove = cb_drag_move;
-	control->onDrop = cb_drop;
-	control->onEnterLeave = cb_enter_leave;
-	control->canRaise = can_raise;
-
-	if (control->isContainer())
-	{
-		((gContainer *)control)->onBeforeArrange = CCONTAINER_cb_before_arrange;
-		((gContainer *)control)->onArrange = CCONTAINER_cb_arrange;
-	}
-	
 	if (control->parent())
 		CCONTAINER_raise_insert((CCONTAINER *)control->parent()->hFree, widget);
 }
 
 CWIDGET *GetContainer(CWIDGET *control)
 {
-	if (!control)
-	{
-		GB.Error("Null container");
+	gContainer *cont;
+	
+	if (GB.CheckObject(control))
 		GB.Propagate();
-	}
-
-	if ( GB.Is (control,CLASS_UserContainer) || GB.Is (control,CLASS_UserControl) )
-		return (CWIDGET*)((CUSERCONTROL*)control)->container;
-
-	return control;
+	
+	cont = ((gContainer *)control->widget)->proxyContainer();
+	//fprintf(stderr, "container: (%p %s)\n", control->widget, control->widget->name());
+	
+	return GetObject(cont);
 }
 
 int CWIDGET_get_handle(void *_object)
@@ -970,6 +952,24 @@ BEGIN_PROPERTY(Control_NoTabFocus)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Control_Direction)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(CONTROL->direction());
+	else
+		CONTROL->setDirection(VPROP(GB_INTEGER));
+
+END_PROPERTY
+
+
+BEGIN_PROPERTY(Control_RightToLeft)
+
+	GB.ReturnBoolean(CONTROL->isRightToLeft());
+
+END_PROPERTY
+
+//---------------------------------------------------------------------------
+
 GB_DESC CWidgetDesc[] =
 {
 	GB_DECLARE("Control", sizeof(CWIDGET)),
@@ -1035,6 +1035,8 @@ GB_DESC CWidgetDesc[] =
 	GB_PROPERTY("PopupMenu", "s", Control_PopupMenu),
 	GB_PROPERTY("Proxy", "Control", Control_Proxy),
 	GB_PROPERTY("NoTabFocus", "b", Control_NoTabFocus),
+	GB_PROPERTY("Direction", "i", Control_Direction),
+	GB_PROPERTY_READ("RightToLeft", "b", Control_RightToLeft),
 
 	GB_PROPERTY_READ("Parent", "Container", Control_Parent),
 	GB_PROPERTY_READ("_Parent", "Container", Control__Parent),
