@@ -114,12 +114,12 @@ static void cb_click(GtkComboBox *widget,gComboBox *data)
 			gtk_entry_set_text(GTK_ENTRY(data->entry), text);
 			data->setIndex(index);
 			data->unlock();
-			data->emit(SIGNAL(data->onChange));
+			CB_textbox_change(data);
 		}
 	}
 
 	if (data->index() >= 0)
-		data->emit(SIGNAL(data->onClick));
+		CB_combobox_click(data);
 }
 
 /*static int pathToIndex(GtkTreePath *path)
@@ -326,6 +326,7 @@ void gComboBox::create(bool readOnly)
 		g_free(save);
 		
 		updateDesign();
+		updateDirection();
 	}
 	
 	unlock();
@@ -333,10 +334,6 @@ void gComboBox::create(bool readOnly)
 
 gComboBox::gComboBox(gContainer *parent) : gTextBox(parent, true)
 {
-	onChange = NULL;
-	onClick = NULL;
-	onActivate = NULL;
-	
 	_no_background = TRUE;
 	_last_key = 0;
 	_model_dirty = false;
@@ -374,6 +371,11 @@ void gComboBox::popup()
 	gtk_combo_box_popup(GTK_COMBO_BOX(widget));
 }
 
+void gComboBox::popdown()
+{
+	gtk_combo_box_popdown(GTK_COMBO_BOX(widget));
+}
+
 #ifdef GTK3
 
 GtkWidget *gComboBox::getStyleSheetWidget()
@@ -521,7 +523,7 @@ void gComboBox::setIndex(int vl)
 		
 	if (vl == index() && vl >= 0)
 	{
-		emit(SIGNAL(onClick));
+		CB_combobox_click(this);
 		return;
 	}
 	
@@ -799,3 +801,22 @@ void gComboBox::setDesign(bool ignore)
 	if (entry)
 		gtk_widget_set_can_focus(entry, false);
 }
+
+static void cb_update_direction(GtkWidget *widget, gpointer data)
+{
+	gtk_widget_set_direction(widget, (GtkTextDirection)(uintptr_t)data);
+	
+	if (GTK_IS_CONTAINER(widget))
+		gtk_container_forall(GTK_CONTAINER(widget), cb_update_direction, data);
+}
+
+void gComboBox::updateDirection()
+{
+	GtkTextDirection dir;
+	
+	gControl::updateDirection();
+	
+	dir = gtk_widget_get_direction(widget);
+	cb_update_direction(widget, (gpointer)(uintptr_t)dir);
+}
+
