@@ -41,6 +41,7 @@
 #include <QWidget>
 #include <QTextCodec>
 
+#include "gb.form.const.h"
 #include "CWidget.h"
 #include "CImage.h"
 #include "CClipboard.h"
@@ -48,6 +49,7 @@
 CDRAG_INFO CDRAG_info = { 0 };
 bool CDRAG_dragging = false;
 void *CDRAG_destination = 0;
+static char CDRAG_action = DRAG_MOVE;
 
 static CPICTURE *_picture = 0;
 static int _picture_x = -1;
@@ -394,44 +396,22 @@ GB_DESC CClipboardDesc[] =
 	GB_END_DECLARE
 };
 
-/** Drag frame ***********************************************************/
 
-//MyDragFrame::MyDragFrame() : QWidget(0, 0, Qt::WType_TopLevel | Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_StaysOnTop | Qt::WX11BypassWM)
-MyDragFrame::MyDragFrame(QWidget *parent) : 
-	QWidget(parent, Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint)
-{
-	setAutoFillBackground(true);
-	QPalette pal(palette());
-	pal.setColor(QPalette::Window, QColor(0, 0, 0, 128));
-	setPalette(pal);
-	//setWindowOpacity(0.5);
-}
-
-/*MyDragFrame::paintEvent(QPaintEvent *e)
-{
-	
-}*/
-
-static QWidget *_frame[4] = { 0 };
-static bool _frame_visible = false;
-static CWIDGET *_frame_control =0;
+//---------------------------------------------------------------------------
 
 static void hide_frame(CWIDGET *control)
 {
-	int i;
+	static GB_FUNCTION func;
+	static bool init = FALSE;
 	
-	if (!_frame_visible)
-		return;
-		
-	if (control && control != _frame_control)
-		return;
-		
-	for (i = 0; i < 4; i++)
-		delete _frame[i];
-		
-	_frame_visible = false;
-	GB.Unref(POINTER(&_frame_control));
-	_frame_control = NULL;
+	if (!init)
+	{
+		GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_HideDNDFrame", NULL, NULL);
+		init = TRUE;
+	}
+	
+	GB.Push(1, GB_T_OBJECT, control);
+	GB.Call(&func, 1, FALSE);
 }
 
 void CDRAG_hide_frame(CWIDGET *control)
@@ -441,54 +421,17 @@ void CDRAG_hide_frame(CWIDGET *control)
 
 static void show_frame(CWIDGET *control, int x, int y, int w, int h)
 {
-	QWidget *wid;
-	//QPoint p = wid->mapToGlobal(QPoint(0, 0));
-	int i;
+	static GB_FUNCTION func;
+	static bool init = FALSE;
 	
-	if (GB.Is(control, CLASS_Container))
-		wid = QCONTAINER(control);
-	else
-		wid = QWIDGET(control);
-	
-	if (w <= 0 || h <= 0)
+	if (!init)
 	{
-		x = y = 0;
-		w = wid->width();
-		h = wid->height();
+		GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_ShowDNDFrame", NULL, NULL);
+		init = TRUE;
 	}
 	
-	//x += p.x();
-	//y += p.y();
-	
-	if (control != _frame_control)
-	{
-		hide_frame(NULL);
-		_frame_control = control;
-		GB.Ref(control);
-	}
-	
-	if (!_frame_visible)
-	{
-		for (i = 0; i < 4; i++)
-			_frame[i] = new MyDragFrame(wid);
-	}
-	
-	//x -= 2;
-	//y -= 2;
-	//w += 4;
-	//h += 4;
-	if (w < 4 || h < 4)
-		return;
-	
-	_frame[0]->setGeometry(x, y, w, 2);
-	_frame[3]->setGeometry(x, y + h - 2, w, 2);
-	_frame[1]->setGeometry(x, y + 2, 2, h - 4);
-	_frame[2]->setGeometry(x + w - 2, y + 2, 2, h - 4);
-	
-	for (i = 0; i < 4; i++)
-		_frame[i]->show();
-	
-	_frame_visible = true;
+	GB.Push(5, GB_T_OBJECT, control, GB_T_INTEGER, x, GB_T_INTEGER, y, GB_T_INTEGER, w, GB_T_INTEGER, h);
+	GB.Call(&func, 5, FALSE);
 }
 
 
@@ -574,9 +517,10 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, GB_STRING *fmt)
 	
 	GB.Unref(POINTER(&CDRAG_destination));
 	CDRAG_destination = 0;
+	CDRAG_action = DRAG_MOVE;
 	
 	//qDebug("start drag");
-	drag->exec();
+	drag->exec(Qt::CopyAction | Qt::MoveAction | Qt::LinkAction);
 
 	source->flag.dragging = false;
 	//qDebug("end drag");
@@ -599,20 +543,38 @@ _BAD_FORMAT:
 }
 
 
+static void update_action(QDropEvent *e)
+{
+	Qt::KeyboardModifiers mod = e->keyboardModifiers() & (Qt::ShiftModifier | Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier);
+	
+	if (mod == Qt::ControlModifier)
+	{
+		e->setDropAction(Qt::CopyAction);
+		CDRAG_action = DRAG_COPY;
+	}
+	else if (mod == Qt::ShiftModifier)
+	{
+		e->setDropAction(Qt::LinkAction);
+		CDRAG_action = DRAG_LINK;
+	}
+	else
+	{
+		e->setDropAction(Qt::MoveAction);
+		CDRAG_action = DRAG_MOVE;
+	}
+}
+
+
 bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 {
 	bool cancel;
 
-	//qDebug("CDRAG_drag_enter: (%s %p) %p", GB.GetClassName(control), control, qobject_cast<MyListView *>(QWIDGET(control)));
-
-	// Hack for QScrollView
-	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
-		((MyListView *)QWIDGET(control))->contentsDragEnterEvent((QDragEnterEvent *)e);*/
-
+	update_action(e);
+	
 	if (!GB.CanRaise(control, EVENT_Drag))
 	{
 		if (GB.CanRaise(control, EVENT_DragMove) || GB.CanRaise(control, EVENT_Drop))
-			e->acceptProposedAction();
+			e->accept();
 		else
 		{
 			if (qobject_cast<QLineEdit *>(w) || qobject_cast<QTextEdit *>(w))
@@ -623,6 +585,8 @@ bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 		return true;
 	}
 	
+	//fprintf(stderr, "CDRAG_drag_enter: %s %s\n", GB.GetClassName(control), control->name);
+
 	CDRAG_clear(true);
 	CDRAG_info.event = e;
 
@@ -633,7 +597,7 @@ bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 	if (cancel)
 		e->ignore();
 	else
-		e->acceptProposedAction();
+		e->accept();
 	return cancel;
 }
 
@@ -641,6 +605,8 @@ bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 
 void CDRAG_drag_leave(CWIDGET *control)
 {
+	//fprintf(stderr, "CDRAG_drag_leave: %s %s\n", GB.GetClassName(control), control->name);
+
 	CDRAG_hide_frame(control);
 	
 	//while (EXT(control) && EXT(control)->proxy)
@@ -665,27 +631,13 @@ bool CDRAG_drag_move(QWidget *w, CWIDGET *control, QDropEvent *e)
 
 	//qDebug("CDRAG_drag_move: (%s %p) %p", GB.GetClassName(control), control, qobject_cast<MyListView *>(QWIDGET(control)));
 
-	// Hack for QScrollView
-	
-	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
-	{
-		accepted = e->isAccepted();
-		((MyListView *)QWIDGET(control))->contentsDragMoveEvent((QDragMoveEvent *)e);
-		if (accepted)
-			e->acceptProposedAction();
-		else
-			e->ignore();
-	}*/
+	update_action(e);
 
 	if (!GB.CanRaise(control, EVENT_DragMove))
-	{
-		/*if (GB.CanRaise(control, EVENT_Drop))
-			e->accept();
-		else
-			e->ignore();*/
 		return true;
-	}
-
+	
+	//fprintf(stderr, "CDRAG_drag_move: %s %s\n", GB.GetClassName(control), control->name);
+	
 	CDRAG_clear(true);
 	CDRAG_info.event = e;
 
@@ -698,9 +650,10 @@ bool CDRAG_drag_move(QWidget *w, CWIDGET *control, QDropEvent *e)
 	if (cancel)
 		e->ignore();
 	else
-		e->acceptProposedAction();
+		e->accept();
 
 	CDRAG_clear(false);
+	
 	return cancel;
 }
 
@@ -717,6 +670,8 @@ bool CDRAG_drag_drop(QWidget *w, CWIDGET *control, QDropEvent *e)
 	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
 		((MyListView *)QWIDGET(control))->contentsDropEvent((QDragMoveEvent *)e);*/
 	
+	//fprintf(stderr, "CDRAG_drag_drop: %s %s\n", GB.GetClassName(control), control->name);
+
 	CDRAG_clear(true);
 	CDRAG_info.event = e;
 	CDRAG_destination = control;
@@ -746,6 +701,16 @@ BEGIN_METHOD(Drag_call, GB_OBJECT source; GB_VARIANT data; GB_STRING format)
 
 END_METHOD
 
+BEGIN_PROPERTY(Drag_Target)
+
+	if (READ_PROPERTY)
+		GB.ReturnObject(CDRAG_destination);
+	else
+		GB.StoreObject(PROP(GB_OBJECT), POINTER(&CDRAG_destination));
+
+END_PROPERTY
+
+
 BEGIN_METHOD_VOID(Drag_exit)
 
 	hide_frame(NULL);
@@ -832,7 +797,7 @@ BEGIN_PROPERTY(Drag_Data)
 END_PROPERTY
 
 
-BEGIN_METHOD(CDRAG_paste, GB_STRING format)
+BEGIN_METHOD(Drag_Paste, GB_STRING format)
 
 	if (!CDRAG_info.valid)
 	{
@@ -848,21 +813,7 @@ END_METHOD
 BEGIN_PROPERTY(Drag_Action)
 
 	CHECK_VALID();
-
-	switch(CDRAG_info.event->dropAction())
-	{
-		case Qt::LinkAction:
-			GB.ReturnInteger(1);
-			break;
-
-		case Qt::MoveAction:
-			GB.ReturnInteger(2);
-			break;
-
-		default:
-			GB.ReturnInteger(0);
-			break;
-	}
+	GB.ReturnInteger(CDRAG_action);
 
 END_PROPERTY
 
@@ -880,7 +831,10 @@ BEGIN_PROPERTY(Drag_X)
 
 	CHECK_VALID();
 
-	GB.ReturnInteger(CDRAG_info.x);
+	if (READ_PROPERTY)
+		GB.ReturnInteger(CDRAG_info.x);
+	else
+		CDRAG_info.x = VPROP(GB_INTEGER);
 
 END_PROPERTY
 
@@ -889,10 +843,14 @@ BEGIN_PROPERTY(Drag_Y)
 
 	CHECK_VALID();
 
-	GB.ReturnInteger(CDRAG_info.y);
+	if (READ_PROPERTY)
+		GB.ReturnInteger(CDRAG_info.y);
+	else
+		CDRAG_info.y = VPROP(GB_INTEGER);
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Drag_Pending)
 
 	GB.ReturnBoolean(CDRAG_dragging);
@@ -932,9 +890,9 @@ GB_DESC CDragDesc[] =
 	GB_CONSTANT("Text", "i", MIME_TEXT),
 	GB_CONSTANT("Image", "i", MIME_IMAGE),
 
-	GB_CONSTANT("Copy", "i", 0),
-	GB_CONSTANT("Link", "i", 1),
-	GB_CONSTANT("Move", "i", 2),
+	GB_CONSTANT("Copy", "i", DRAG_COPY),
+	GB_CONSTANT("Link", "i", DRAG_LINK),
+	GB_CONSTANT("Move", "i", DRAG_MOVE),
 
 	GB_STATIC_PROPERTY("Icon", "Picture", Drag_Icon),
 	GB_STATIC_PROPERTY("IconX", "i", Drag_IconX),
@@ -946,15 +904,16 @@ GB_DESC CDragDesc[] =
 	GB_STATIC_PROPERTY_READ("Type", "i", Drag_Type),
 	GB_STATIC_PROPERTY_READ("Action", "i", Drag_Action),
 	GB_STATIC_PROPERTY_READ("Source", "Control", Drag_Source),
-	GB_STATIC_PROPERTY_READ("X", "i", Drag_X),
-	GB_STATIC_PROPERTY_READ("Y", "i", Drag_Y),
+	GB_STATIC_PROPERTY("X", "i", Drag_X),
+	GB_STATIC_PROPERTY("Y", "i", Drag_Y),
 	GB_STATIC_PROPERTY_READ("Pending", "b", Drag_Pending),
+	GB_STATIC_PROPERTY("_Target", "Control", Drag_Target),
 
 	GB_STATIC_METHOD("_call", "Control", Drag_call, "(Source)Control;(Data)v[(Format)s]"),
 	GB_STATIC_METHOD("_exit", NULL, Drag_exit, NULL),
 	GB_STATIC_METHOD("Show", NULL, Drag_Show, "(Control)Control;[(X)i(Y)i(Width)i(Height)i]"),
 	GB_STATIC_METHOD("Hide", NULL, Drag_Hide, NULL),
-	GB_STATIC_METHOD("Paste", "v", CDRAG_paste, "[(Format)s]"),
+	GB_STATIC_METHOD("Paste", "v", Drag_Paste, "[(Format)s]"),
 
 	GB_END_DECLARE
 };
