@@ -244,7 +244,7 @@ static void handle_focus(CWINDOW *_object)
 	if (THIS->focus)
 	{
 		//qDebug("handle_focus on %s", THIS->focus->name);
-		THIS->focus->widget->setFocus();
+		CWIDGET_set_focus(THIS->focus);
 		GB.Unref(POINTER(&THIS->focus));
 		THIS->focus = NULL;
 	}
@@ -676,21 +676,6 @@ static bool do_close(CWINDOW *_object, int ret, bool destroyed = false)
 		#endif
 	}
 
-	#if 0
-	if (closed || destroyed)
-	{
-		if (CWINDOW_Active == THIS)
-			CWINDOW_activate(CWidget::get(WIDGET->parentWidget()));
-		if (CWINDOW_LastActive == THIS)
-		{
-			//GB.Unref(POINTER(&CWINDOW_LastActive));
-			CWINDOW_LastActive = 0;
-			//qDebug("CWINDOW_LastActive = 0");
-		}
-		THIS->opened = FALSE;
-	}
-	#endif
-
 	if (closed)
 		THIS->ret = ret;
 
@@ -1969,11 +1954,7 @@ void MyMainWindow::doShowModal(bool popup, const QPoint *pos)
 
 		parent = CWINDOW_Current;
 		if (!parent)
-		{
-			parent = CWINDOW_Main;
-			if (!parent)
-				parent = CWINDOW_Active;
-		}
+			parent = CWINDOW_Active;
 
 		if (parent)
 			setParent(CWidget::getTopLevel((CWIDGET *)parent)->widget.widget, Qt::Window | info.flags);
@@ -1987,6 +1968,9 @@ void MyMainWindow::doShowModal(bool popup, const QPoint *pos)
 	setEventLoop();
 
 	THIS->loopLevel++;
+	
+	THIS->save_focus = CWIDGET_active_control;
+	THIS->previous = CWINDOW_Current;
 	CWINDOW_Current = THIS;
 
 	_enterLoop = true;
@@ -1998,9 +1982,9 @@ void MyMainWindow::doShowModal(bool popup, const QPoint *pos)
 
 	GB.OnErrorBegin(&handler);
 
-	//fprintf(stderr, "eventLoop.exec <---- %p\n",CWINDOW_Current);
+	//fprintf(stderr, "event loop <---- %p\n",CWINDOW_Current);
 	eventLoop.exec();
-	//fprintf(stderr, "eventLoop.exec ---->%p\n", info.save);
+	//fprintf(stderr, "event loop ---->%p\n", info.save);
 
 	GB.OnErrorEnd(&handler);
 
@@ -2022,6 +2006,12 @@ void MyMainWindow::doShowModal(bool popup, const QPoint *pos)
 		CWIDGET_leave_popup(info.save_popup);
 	
 	CWINDOW_ensure_active_window();
+	
+	if (THIS->save_focus)
+	{
+		CWIDGET_set_focus(THIS->save_focus);
+		THIS->save_focus = NULL;
+	}
 }
 
 #if 0
@@ -2232,18 +2222,30 @@ void MyMainWindow::setBorder(bool b)
 	if (!isWindow())
 		return;
 
-	//qDebug("effectiveWinId");
+#ifdef QT5
+	
+	bool visible = isVisible();
+	void *_object = CWidget::get(this);;
+	
+	if (_border)
+		setWindowFlags(windowFlags() & ~Qt::FramelessWindowHint);
+	else
+		setWindowFlags(windowFlags() | Qt::FramelessWindowHint);
+	
+	if (visible) 
+	{
+		show();
+		move(THIS->x, THIS->y);
+	}
+
+#else
+
 	initProperties(PROP_BORDER);
-	#ifdef QT5
-		PLATFORM.Window.Remap(WINDOW);
-	#else
 	if (effectiveWinId())
 		X11_window_remap(effectiveWinId());
-	#endif
-
-	#ifndef QT5
 	doReparent(parentWidget(), pos());
-	#endif
+	
+#endif
 }
 
 void MyMainWindow::setResizable(bool b)
@@ -2259,8 +2261,6 @@ void MyMainWindow::setResizable(bool b)
 
 void MyMainWindow::setUtility(bool b)
 {
-	Qt::WindowFlags flags;
-
 	if (_utility == b)
 		return;
 
@@ -2546,11 +2546,6 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
 	THIS->closed = true;
 	//qApp->sendEvent(WIDGET, new QEvent(EVENT_CLOSE));
 
-	/*if (CWINDOW_Active == THIS)
-	{
-		//qDebug("closeEvent activate: %p %p", CWidget::get(WIDGET->parentWidget()), CWINDOW_Active);
-		CWINDOW_activate(CWidget::get(WIDGET->parentWidget()));
-	}*/
 	if (CWINDOW_LastActive == THIS)
 	{
 		//GB.Unref(POINTER(&CWINDOW_LastActive));
@@ -2933,8 +2928,13 @@ bool MyMainWindow::focusNextPrevChild(bool next)
 		if (!current || current == initial)
 			return QWidget::focusNextPrevChild(next);
 		
+		//fprintf(stderr, "focusNextPrevChild: %s / %d\n", ((CWIDGET *)current)->name, CWIDGET_has_no_tab_focus(current));
+		
+		if (CWIDGET_has_no_tab_focus(current))
+			continue;
+		
 		w = QWIDGET(current);
-		if (w->isVisible() && w->isEnabled() && (w->focusPolicy() != Qt::NoFocus)) // & Qt::TabFocus))
+		if (w->isVisible() && w->isEnabled() && (w->focusPolicy() & Qt::TabFocus))
 		{
 			CWIDGET_set_focus(current);
 			return true;
@@ -2969,8 +2969,22 @@ void CWINDOW_activate(CWIDGET *ob)
 {
 	CWINDOW *active;
 
-	//qDebug("CWINDOW_activate: %s", ob ? ob->name : NULL);
-
+	/*if (ob)
+	{
+		fprintf(stderr, "CWINDOW_activate: %s %s\n", GB.GetClassName(ob), ob->name);
+		
+		CWIDGET *parent = ob;
+		for(;;)
+		{
+			parent = (CWIDGET *)CWIDGET_get_parent(parent);
+			if (!parent)
+				break;
+			fprintf(stderr, "  --> %s %s\n", GB.GetClassName(parent), parent->name);
+		}
+	}
+	else
+		fprintf(stderr, "CWINDOW_activate: null\n");*/
+	
 	if (ob)
 	{
 		active = CWidget::getWindow(ob);
@@ -2998,7 +3012,10 @@ void CWINDOW_activate(CWIDGET *ob)
 	}
 
 	if (active)
+	{
+		//fprintf(stderr, "Activate: %s\n", GB.GetClassName(active));
 		GB.Raise(active, EVENT_Activate, 0);
+	}
 
 	CWINDOW_Active = active;
 	
@@ -3050,6 +3067,8 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 		{
 			MyMainWindow *w = (MyMainWindow *)o;
 
+			THIS->noHideEvent = false;
+			
 			if (THIS->toplevel && !THIS->popup && !THIS->moved)
 				w->center();
 
@@ -3067,9 +3086,10 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 		}
 		else if (e->type() == QEvent::Hide) // && !e->spontaneous())
 		{
-			//qDebug("Hide: %s %d (%d)", GB.GetClassName(THIS), WINDOW->isHidden(), e->spontaneous());
-			//if (WINDOW->isHidden())
+			//fprintf(stderr, "Hide: %p %s %d (%d)\n", o, GB.GetClassName(THIS), WINDOW->isHidden(), e->spontaneous());
+			if (!THIS->noHideEvent)
 			{
+				THIS->noHideEvent = true;
 				GB.Raise(THIS, EVENT_Hide, 0);
 				if (!e->spontaneous())
 					CACTION_raise(THIS);
