@@ -423,7 +423,7 @@ static bool QT_EventFilter(QEvent *e)
 			GB.FreeString(&CKEY_info.text);
 			//qDebug("IMEnd: %s", imevent->text().latin1());
 			CKEY_info.text = GB.NewZeroString(QT_ToUtf8(imevent->commitString()));
-			CKEY_info.state = 0;
+			CKEY_info.state = Qt::KeyboardModifiers();
 			CKEY_info.code = 0;
 		}
 	}
@@ -842,7 +842,7 @@ static bool must_quit(void)
 	#if DEBUG_WINDOW
 	qDebug("must_quit: Window = %d Watch = %d in_event_loop = %d MAIN_in_message_box = %d _prevent_quit = %d", CWindow::count, CWatch::count, in_event_loop, MAIN_in_message_box, _prevent_quit);
 	#endif
-	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0 && _prevent_quit == 0;
+	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0 && _prevent_quit == 0 && !GB.HasActiveTimer();
 }
 
 static void check_quit_now(intptr_t param)
@@ -950,8 +950,7 @@ __INSTALL_TRANSLATOR:
 	qApp->installTranslator(_translator);
 
 __SET_DIRECTION:
-	if (rtl)
-		qApp->setLayoutDirection(Qt::RightToLeft);
+	qApp->setLayoutDirection(rtl ? Qt::RightToLeft : Qt::LeftToRight);
 }
 
 static void hook_lang(char *lang, int rtl)
@@ -1082,6 +1081,8 @@ static void hook_timer(GB_TIMER *timer, bool on)
 
 	if (on)
 		timer->id = (intptr_t)(new MyTimer(timer));
+	else
+		MAIN_check_quit();
 }
 
 
@@ -1356,7 +1357,7 @@ extern "C" {
 GB_DESC *GB_CLASSES[] EXPORT =
 {
 	BorderDesc, CColorDesc,
-	AlignDesc, ArrangeDesc, ScrollDesc, CKeyDesc, SelectDesc,
+	AlignDesc, ArrangeDesc, ScrollDesc, CKeyDesc, SelectDesc, DirectionDesc,
 	CImageDesc, CPictureDesc, AnimationDesc,
 	CFontDesc, CFontsDesc,
 	CMouseDesc, CCursorDesc, CPointerDesc,
