@@ -272,9 +272,8 @@ static void resize_container(void *_object, QWidget *cont, int w, int h)
 #define CONTAINER_TYPE QWidget *
 #define ARRANGEMENT_TYPE CCONTAINER_ARRANGEMENT *
 
-#define IS_RIGHT_TO_LEFT() qApp->isRightToLeft()
-
 #define GET_WIDGET(_object) ((CWIDGET *)_object)->widget
+#define IS_RIGHT_TO_LEFT(_object) (((CWIDGET *)_object)->widget)->isRightToLeft()
 #define GET_CONTAINER(_object) ((CCONTAINER *)_object)->container
 #define GET_ARRANGEMENT(_object) ((CCONTAINER_ARRANGEMENT *)_object)
 #define IS_EXPAND(_object) (((CWIDGET *)_object)->flag.expand)
@@ -897,6 +896,9 @@ void MyContainer::changeEvent(QEvent *e)
 {
 	void *_object = CWidget::get(this);
 
+	if (e->type() == QEvent::LayoutDirectionChange)
+		CCONTAINER_arrange(THIS);
+	
 	if (!THIS_ARRANGEMENT->paint)
 	{
 		MyFrame::changeEvent(e);
@@ -1333,26 +1335,27 @@ BEGIN_PROPERTY(UserControl_Container)
 		}
 
 		if (!p)
-			GB.Error("Container must be a child control");
-		else
 		{
-			GB_COLOR bg = CWIDGET_get_background((CWIDGET *)current, true);
-			GB_COLOR fg = CWIDGET_get_foreground((CWIDGET *)current, true);
-
-			if (current)
-				CWIDGET_container_for(current, NULL);
-			CWIDGET_container_for(cont, THIS);
-			
-			THIS->container = w;
-
-			CCONTAINER_arrange(THIS);
-
-			CWIDGET_set_color((CWIDGET *)cont, bg, fg, true);
-			
-			CCONTAINER_update_design(THIS);
-
-			CWIDGET_register_proxy(THIS, cont);
+			GB.Error("Container must be a child control");
+			return;
 		}
+		
+		GB_COLOR bg = CWIDGET_get_background((CWIDGET *)current, true);
+		GB_COLOR fg = CWIDGET_get_foreground((CWIDGET *)current, true);
+
+		if (current)
+			CWIDGET_container_for(current, NULL);
+		CWIDGET_container_for(cont, THIS);
+		
+		THIS->container = w;
+
+		CCONTAINER_arrange(THIS);
+
+		CWIDGET_set_color((CWIDGET *)cont, bg, fg, true);
+		
+		CCONTAINER_update_design(THIS);
+
+		CWIDGET_register_proxy(THIS, cont);
 	}
 
 END_PROPERTY
