@@ -37,6 +37,7 @@
 
 DECLARE_EVENT(EVENT_Change);
 DECLARE_EVENT(EVENT_Activate);
+DECLARE_EVENT(EVENT_Cursor);
 DECLARE_EVENT(EVENT_Click);
 
 #define MAX_LEN 32767
@@ -83,6 +84,7 @@ BEGIN_METHOD(TextBox_new, GB_OBJECT parent)
 
 	QObject::connect(wid, SIGNAL(textChanged(const QString &)), &CTextBox::manager, SLOT(onChange()));
 	QObject::connect(wid, SIGNAL(returnPressed()), &CTextBox::manager, SLOT(onActivate()));
+	QObject::connect(wid, SIGNAL(cursorPositionChanged(int, int)), &CTextBox::manager, SLOT(onCursor()));
 	//QObject::connect(wid, SIGNAL(selectionChanged()), &CTextBox::manager, SLOT(onSelectionChanged()));
 
 	wid->setAlignment(Qt::AlignLeft);
@@ -457,6 +459,7 @@ static void combo_set_editable(void *_object, bool ed)
 		COMBOBOX->setCompleter(0);
 		//CWidget::installFilter(COMBOBOX);
 		QObject::connect(COMBOBOX->lineEdit(), SIGNAL(returnPressed()), &CTextBox::manager, SLOT(onActivate()));
+		QObject::connect(COMBOBOX->lineEdit(), SIGNAL(cursorPositionChanged(int, int)), &CTextBox::manager, SLOT(onCursor()));
 		//QObject::connect(COMBOBOX->lineEdit(), SIGNAL(selectionChanged()), &CTextBox::manager, SLOT(onSelectionChanged()));
 
 		if (CWIDGET_is_design(THIS))
@@ -558,6 +561,12 @@ BEGIN_METHOD_VOID(ComboBox_Popup)
 
 END_METHOD
 
+BEGIN_METHOD_VOID(ComboBox_Close)
+
+	COMBOBOX->hidePopup();
+
+END_METHOD
+
 
 BEGIN_PROPERTY(ComboBox_Text)
 
@@ -834,6 +843,12 @@ void CTextBox::onClick()
 }
 
 
+void CTextBox::onCursor()
+{
+	RAISE_EVENT(EVENT_Cursor);
+}
+
+
 /*void CTextBox::onSelectionChanged(void)
 {
 	GET_SENDER();
@@ -907,6 +922,7 @@ GB_DESC CTextBoxDesc[] =
 
 	GB_EVENT("Change", NULL, NULL, &EVENT_Change),
 	GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
+	GB_EVENT("Cursor", NULL, NULL, &EVENT_Cursor),
 
 	TEXTBOX_DESCRIPTION,
 
@@ -947,6 +963,7 @@ GB_DESC CComboBoxDesc[] =
 	GB_PROPERTY_READ("Selected", "b", ComboBox_Selected),
 
 	GB_METHOD("Popup", NULL, ComboBox_Popup, NULL),
+	GB_METHOD("Close", NULL, ComboBox_Close, NULL),
 	GB_METHOD("Clear", NULL, ComboBox_Clear, NULL),
 	GB_METHOD("Insert", NULL, TextBox_Insert, "(Text)s"),
 
@@ -968,6 +985,7 @@ GB_DESC CComboBoxDesc[] =
 
 	GB_EVENT("Change", NULL, NULL, &EVENT_Change),
 	GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
+	GB_EVENT("Cursor", NULL, NULL, &EVENT_Cursor),
 	GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
 	COMBOBOX_DESCRIPTION,
